/***************************************************************************

	File: W5100.h

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

*/

#ifndef	__W5100_H
#define	__W5100_H

// uncomment don't use 
#define	__USE_FUNCTION
#define	__USE_TCP
#define	__USE_UDP

// IPphone udp only
#undef	__USE_TCP

// use macro instead function 
//#undef	__USE_FUNCTION

#include "typedef.h"

/** DEFINITION ************************************************************/
#define	NONE_BLOCK	1
#define	BLOCK		0
#define	MAX_SOCKET	4

// Common Register
#define	MR		0x0000	// mode register
#define	GWR		0x0001	// Gateway IP address register
#define	SUBR	0x0005	// Subnet Mask register
#define	SHAR	0x0009	// Source Hardware address register
#define	SIPR	0x000f	// Source IP address register
#define	IR		0x0015	// Interrupt register
#define	IMR		0x0016	// Interrupt Mask register
#define	RTR		0x0017	// Retry Time-valu register
#define	RCR		0x0019	// Retry count register
#define	RMSR	0x001a	// RX memory size register(0x55, 2kbyte x 4socket)
#define	TMSR	0x001b	// TX memory size register(0x55, 2kbyte x 4socket)
#define	RATR	0x001c	// Authentication Type in PPPoE mode
#define	PTIMER	0x0028	// PPP LCP request timer register
#define	PMAGIC	0x0029	// PPP LCP magic number register
#define	UIPR	0x002a	// Unreachable IP address register
#define	UPORT	0x002e	// Unreachable Port register

// Socket Register
#define	Sn_MR		0x0400	// socket Mode register
#define	Sn_CR		0x0401	// socket Command register
#define	Sn_IR		0x0402	// socket interrupt register(Read only)
#define	Sn_SR		0x0403	// socket status register(Read only)
#define	Sn_PORT		0x0404	// socket source port register
#define	Sn_DHAR		0x0406	// socket Peer MAC address register
#define Sn_DIPR		0x040c	// socket Peer IP address register
#define Sn_DPORT	0x0410	// socket Peer port register
#define Sn_MSSR		0x0412	// socket Maximum Segment Size register
#define	Sn_PROTO	0x0414	// socket Protocol of IP Header field register in IP raw mode
#define	Sn_TOS		0x0415	// socket IP Type of Service(TOS) Register 
#define	Sn_TTL		0x0416	// socket IP Time to live(TTL) Register
#define	Sn_TX_FSR 	0x0420	// socket Transmit free memory size register(Read only)
#define	Sn_TX_RD 	0x0422	// socket Transmit memory read pointer register
#define	Sn_TX_WR	0x0424	// socket Transmit memory write pointer register
#define	Sn_RX_RSR	0x0426	// socket Received data size register(Read only)
#define	Sn_RX_RD	0x0428	// socket Read point of Receive memory 
#define	Sn_RX_WR	0x042a	// socket Write point of Receive memory 

// Socket Register area size
#define	SOCKET_REGISTER_SIZE	0x0100

// Memory base addr, Mmemory size
#define	RX_MEMORY_BASE_ADDR		0x6000	// RX memory base address
#define	TX_MEMORY_BASE_ADDR		0x4000	// TX memory base address
#define	RX_MEMORY_SIZE			0x800	// RX memory size per socket is 2048bytes(0x800).
#define	TX_MEMORY_SIZE			0x800	// TX memory size per socket is 2048bytes(0x800).

// MODE register values 
#define MR_RST			0x80	// reset 
#define MR_PB			0x10	// ping block 
#define MR_PPPOE		0x08	// enable pppoe 
#define MR_LB  			0x04	// little or big endian selector in indirect mode 
#define MR_AI			0x02	// auto-increment in indirect mode 
#define MR_IND			0x01	// enable indirect mode 

// IR register values 
#define IR_CONFLICT		0x80	// check ip confict 
#define IR_UNREACH		0x40	// get the destination unreachable message in UDP sending 
#define IR_PPPoE		0x20	// get the PPPoE close message 
#define IR_S3_INT		0x08	// check socket3 interrupt 
#define IR_S2_INT		0x04	// check socket2 interrupt 
#define IR_S1_INT		0x02	// check socket1 interrupt 
#define IR_S0_INT		0x01	// check socket0 interrupt 

// Socket mode register(Sn_MR) values 
#define SOCK_MR_CLOSE	0x00	// unused socket 
#define SOCK_MR_TCP		0x01	// TCP 
#define SOCK_MR_UDP		0x02	// UDP 
#define SOCK_MR_IPRAW	0x03	// IP LAYER RAW SOCK 
#define SOCK_MR_MACRAW	0x04	// MAC LAYER RAW SOCK 
#define SOCK_MR_PPPOE	0x05	// PPPoE 
#define SOCK_MR_ND		0x20	// No Delayed Ack(TCP) flag 
#define SOCK_MR_MULTI	0x80	// support multicating 

// Socket command register(Sn_CR) values 
#define SOCK_CR_OPEN		0x01	// initialize or open socket 
#define SOCK_CR_LISTEN		0x02	// wait connection request in tcp mode(Server mode) 
#define SOCK_CR_CONNECT		0x04	// send connection request in tcp mode(Client mode) 
#define SOCK_CR_DISCON		0x08	// send closing reqeuset in tcp mode 
#define SOCK_CR_CLOSE		0x10	// close socket 
#define SOCK_CR_SEND		0x20	// updata txbuf pointer, send data 
#define SOCK_CR_SEND_MAC	0x21	// send data with MAC address, so without ARP process 
#define SOCK_CR_SEND_KEEP	0x22	//  send keep alive message 
#define SOCK_CR_RECV		0x40	// update rxbuf pointer, recv data 
// PPPoE command
#define SOCK_CR_PCON		0x23		 
#define SOCK_CR_PDISCON		0x24		 
#define SOCK_CR_PCR			0x25		 
#define SOCK_CR_PCN			0x26		
#define SOCK_CR_PCJ			0x27		

// Socket interrupt register(Sn_IR) values 
#define SOCK_IR_SEND_OK	0x10	// complete sending 
#define SOCK_IR_TIMEOUT	0x08	// assert timeout 
#define SOCK_IR_RECV	0x04	// receiving data 
#define SOCK_IR_DISCON	0x02	// closed socket 
#define SOCK_IR_CON		0x01	// established connection 
// PPPoE interrupt
#define SOCK_IR_PRECV	0x80		
#define SOCK_IR_PFAIL	0x40		
#define SOCK_IR_PNEXT	0x20		

// Socket status register(Sn_SR) values 
#define SOCK_SR_CLOSED		0x00	// closed 
#define SOCK_SR_INIT		0x13	// init state 
#define SOCK_SR_LISTEN		0x14	// listen state 
#define SOCK_SR_SYNSENT		0x15	// connection state 
#define SOCK_SR_SYNRECV		0x16	// connection state 
#define SOCK_SR_ESTABLISHED 0x17	// success to connect 
#define SOCK_SR_FIN_WAIT	0x18	// closing state 
#define SOCK_SR_CLOSING		0x1A	// closing state 
#define SOCK_SR_TIME_WAIT	0x1B	// closing state 
#define SOCK_SR_CLOSE_WAIT	0x1C	// closing state 
#define SOCK_SR_LAST_ACK	0x1D	// closing state 
#define SOCK_SR_UDP			0x22	// udp socket 
#define SOCK_SR_IPRAW		0x32	// ip raw mode socket 
#define SOCK_SR_MACRAW		0x42	// mac raw mode socket 
#define SOCK_SR_PPPOE		0x5F	// pppoe socket 

// Socket IP Protocol Register(Sn_PROTO) values
#define IPPROTO_IP		0		// Dummy for IP 
#define IPPROTO_ICMP	1		// Control message protocol 
#define IPPROTO_IGMP	2		// Internet group management protocol 
#define IPPROTO_GGP		3		// Gateway^2 (deprecated) 
#define IPPROTO_TCP		6		// TCP 
#define IPPROTO_PUP		12		// PUP 
#define IPPROTO_UDP		17		// UDP 
#define IPPROTO_IDP		22		// XNS idp 
#define IPPROTO_ND		77		// UNOFFICIAL net disk protocol 
#define IPPROTO_RAW		255		// Raw IP packet 

/** FUNCTION PROTOTYPE ****************************************************/

//	Common Register Set operation
#ifdef __USE_FUNCTION
	extern	void	SetMR(uchar mode);
	extern	void	SetGWR(uchar ip[]);
	extern	void	SetSUBR(uchar ip[]);
	extern	void	SetSHAR(uchar mac[]);
	extern	void	SetSIPR(uchar ip[]);
	extern	void	SetIMR(uchar mask);
	extern	void	SetRTR(uint val);
	extern	void	SetRCR(uchar count);
	extern	void	SetRMSR(uchar mem_alloc);
	extern	void	SetTMSR(uchar mem_alloc);
	extern	void	SetPATR(uint auth_type);
	extern	void	SetPTIMER(uchar val);
	extern	void	SetPMAGIC(uchar number);
	extern	void	SetUIPR(uchar ip[]);
	extern	void	SetUPORT(uint port);
#else	// __USE_MACROS
	#define	SetMR(mode)		write_byte(MR, mode)
	#define	SetGWR(ip)		write_register(4, GWR, ip)
	#define	SetSUBR(ip)		write_register(4, SUBR, ip)
	#define	SetSHAR(mac)	write_register(6, SHAR, mac)
	#define	SetSIPR(ip)		write_register(4, SIPR, ip)
	#define	SetIMR(mask)	write_byte(IMR, mask)
	#define	SetRTR(val)		write_word(RTR, val)
	#define	SetRCR(count)	write_byte(RCR, count)
	#define	SetRMSR(mem_alloc)	write_byte(RMSR, mem_alloc)
	#define	SetTMSR(mem_alloc)	write_byte(TMSR, mem_alloc)
	#define	SetPATR(auth_type)	write_word(RATR, auth_type)
	#define	SetPTIMER(val)		write_byte(PTIMR, val)
	#define	SetPMAGIC(number)	write_byte(PMAGIC, number) 
	#define	SetUIPR(ip)		write_register(4, UIPR, ip)
	#define	SetUPORT(port)	write_word(UPORT, val)
#endif

//	Common Register Get operation
#ifdef	__USE_FUNCTION
	extern	uchar	GetMR(void);
	extern	void	GetGWR(uchar ip[]);
	extern	void	GetSUBR(uchar ip[]);
	extern	void	GetSHAR(uchar mac[]);
	extern	void	GetSIPR(uchar ip[]);
	extern	uchar	GetIMR(void);
	extern	uint	GetRTR(void);
	extern	uchar	GetRCR(void);
	extern	uchar	GetRMSR(void);
	extern	uchar	GetTMSR(void);
	extern	uint	GetPATR(void);
	extern	uchar	GetPTIMER(void);
	extern	uchar	GetPMAGIC(void);
	extern	void	GetUIPR(uchar ip[]);
	extern	uint	GetUPORT(void);
#else	// __USE_MACRO
	#define	GetMR()		read_byte(MR)
	#define	GetGWR(ip)	read_register(4, GWR, ip)
	#define	GetSUBR(ip)	read_register(4, SUBR, ip)
	#define	GetSHAR(mac) read_register(6, SHAR, mac)
	#define	GetSIPR(ip)	read_register(4, SIPR, ip)
	#define	GetIMR()	read_byte(IMR)
	#define	GetRTR()	read_word(RTR)
	#define	GetRCR()	read_byte(RCR)
	#define	GetRMSR()	read_byte(RMSR)
	#define	GetTMSR()	read_byte(TMSR)
	#define	GetPATR()	read_word(PATR)
	#define	GetPTIMER()	read_byte(PTIMER)
	#define	GetPMAGIC()	read_byte(PMAGIC)
	#define	GetUIPR(ip) read_register(4, UIPR, ip)
	#define	GetUPORT()	read_word(UPORT)
#endif

// Socket Operation function
extern	void clear_socket_flg(void);
extern	int	socket(uchar mode, uint port, uchar ip_proto);
extern	int	listen(int sck_fd);
extern	int	connect(int sck_fd, uchar *to_ip, uint to_port);
extern	int	disconnect(int sck_fd);
extern	int	closesocket(int sck_fd);
extern	int	send(int sck_fd, uchar *buf, uint len, uint flag);
extern	int	recv(int sck_fd, uchar *buf, uint len, uint flag);
extern	int	sendto(int sck_fd, uchar *buf, uint len, uint flag, uchar *to_ip, uint to_port);
extern	int	recvfrom(int sck_fd, uchar *buf, uint len, uint flag, uchar *from_ip, uint *from_port);

//	Socket Register Set Operation MACRO
#define	SetSnMR(sck_fd, mode)		write_byte(Sn_MR + sck_fd * SOCKET_REGISTER_SIZE, mode)
#define	SetSnCR(sck_fd, command)	write_byte(Sn_CR + sck_fd * SOCKET_REGISTER_SIZE, command)
#define	SetSnIR(sck_fd, isr)		write_byte(Sn_IR + sck_fd * SOCKET_REGISTER_SIZE, isr)
#define	SetSnPORT(sck_fd, port)		write_word(Sn_PORT + sck_fd * SOCKET_REGISTER_SIZE, port)
#define	SetSnDHAR(sck_fd, mac)		write_register(6, Sn_DHAR + sck_fd * SOCKET_REGISTER_SIZE, mac) 
#define	SetSnDIPR(sck_fd, ip)		write_register(4, Sn_DIPR + sck_fd * SOCKET_REGISTER_SIZE, ip) 
#define	SetSnDPORT(sck_fd, port)	write_word(Sn_DPORT + sck_fd * SOCKET_REGISTER_SIZE, port)
#define	SetSnMSS(sck_fd, size)		write_word(Sn_MSS + sck_fd * SOCKET_REGISTER_SIZE, size)
#define	SetSnPROTO(sck_fd, proto)	write_byte(Sn_PROTO + sck_fd * SOCKET_REGISTER_SIZE, proto)
#define	SetSnTOS(sck_fd, val)		write_byte(Sn_TOS + sck_fd * SOCKET_REGISTER_SIZE, val)
#define	SetSnTTL(sck_fd, count)		write_byte(Sn_TTL + sck_fd * SOCKET_REGISTER_SIZE, count)
#define	SetSnTXWR(sck_fd, ptr)		write_word(Sn_TX_WR + sck_fd * SOCKET_REGISTER_SIZE, ptr)
#define	SetSnRXRD(sck_fd, ptr)		write_word(Sn_RX_RD + sck_fd * SOCKET_REGISTER_SIZE, ptr)

// Socket Register Get Operation MACRO
#define	GetSnMR(sck_fd)			read_byte(Sn_MR + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnCR(sck_fd)			read_byte(Sn_CR + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnIR(sck_fd)			read_byte(Sn_IR + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnSR(sck_fd)			read_byte(Sn_SR + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnPORT(sck_fd)		read_word(Sn_PORT + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnDHAR(sck_fd, mac)	read_register(6, Sn_DHAR + sck_fd * SOCKET_REGISTER_SIZE, mac)
#define	GetSnDIPR(sck_fd, ip)	read_register(4, Sn_DIPR + sck_fd * SOCKET_REGISTER_SIZE, ip) 
#define	GetSnDPORT(sck_fd)		read_word(Sn_DPORT + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnMSS(sck_fd)		read_word(Sn_MSS + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnPROTO(sck_fd)		read_byte(Sn_PROTO + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnTOS(sck_fd)		read_byte(Sn_TOS + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnTTL(sck_fd)		read_byte(Sn_TTL + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnTXRD(sck_fd)		read_word(Sn_TX_RD + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnTXWR(sck_fd)		read_word(Sn_TX_WR + sck_fd * SOCKET_REGISTER_SIZE)
#define	GetSnRXRD(sck_fd)		read_word(Sn_RX_RD + sck_fd * SOCKET_REGISTER_SIZE)

// Socket Register Get function
extern	uint	GetSnRXRSR(int sck_fd);
extern	uint	GetSnTXFSR(int sck_fd);

// Socket RX/TX memory read write function
extern	void	write_memory(int sck_fd, uint write_ptr, uchar *buf, uint len);
extern	void	read_memory(int sck_fd, uint read_ptr, uchar *buf, uint len);

// W5100 Register/Memory read write basic function
extern	void	write_register(int octet, uint reg_addr, uchar *data);
extern	void	write_word(uint addr, uint data);
extern	void	write_byte(uint addr, uchar data);
extern	void	read_register(int octet, uint reg_addr, uchar *data);
extern	uint	read_word(uint addr);
extern	uchar	read_byte(uint addr);

// other utility
extern	void	bg2lt(uchar val[]);
extern	uint	htons(uint val);
extern	ulong	htonl(ulong val);
extern	uint	ntohs(uint val);
extern	ulong	ntohl(ulong val);
extern	char	*print_ip(char *strbuf, uchar ip[]);
extern	char	*print_mac(char *strbuf, uchar mac[]);
extern	void	set_ip(uchar ip[], char *ip_str);
extern	void	set_mac(uchar mac[], char *mac_str);

#endif	// __W5100_H

/*** end of W5100.h *******************************************************/
