/***************************************************************************

	dtmf.c

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

	This file is part of IPphone.
	generate DTMF dial tone

***************************************************************************/

/** INCLUDES **************************************************************/

/** Variable **************************************************************/
/*
 PB tone/Dial number mapping table
	    1209  1336  1477  Hz
	697    1     2     3
	770    4     5     6
	852    7     8     9
	941    *     0     #

 Tone signal sampled by 8Khz.
	 freq.  ** samples *****  real frequency error
	  697	2cycles 23sample  16,000/23= 696 -0.2% 
	  770 	3cycles	31sample  24,000/31= 774 +0.5%
	  852 	3cycles 28sample  24,000/28= 857 +0.6%
	  941 	2cycles 17sample  16,000/17= 941  0%
	 1209 	5cycles	33sample  40,000/33=1212 +0.3%
	 1336 	4cycles	24sample  32,000/24=1333 -0.2%
	 1477 	5cycles	27sample  40,000/27=1481 +0.3%
*/

// tone wave data
const char freq697[] = {	// 696Hz 2cycle
	  62,  107,  120,   98,   48, 
	 -16,  -76, -113, -117,  -88, 
	 -32,   32,   88,  117,  113, 
	  76,   16,  -48,  -98, -120, 
	-107,  -62,    0, 
	};

const char freq770[] = { // 774Hz 3cycle
	  69,  112,  116,   78,   12, 
	 -58, -108, -119,  -87,  -25, 
	  47,  101,  120,   95,   37, 
	 -35,  -94, -120, -102,  -48, 
	  23,   86,  118,  108,   59, 
	 -11,  -77, -116, -113,  -70, 
	  0, 
	};

const char freq852[] = { // 857Hz 3cycle
	  75,  117,  108,   52,  -26, 
	 -94, -120,  -94,  -27,   52, 
	 108,  117,   75,    1,  -74, 
	-117, -109,  -53,   26,   93, 
	 120,   95,   28,  -51, -108, 
	-117,  -76,   0, 
	};

const char freq941[] = { // 941Hz 2cycle
	  81,  119,   96,   22,  -63, 
	-115, -108,  -44,   43,  107, 
	 116,   64,  -21,  -95, -120, 
	 -82,   0, 
	};

const char freq1209[] = { // 1212Hz 5cycle
	  98,  113,   34,  -74, -120, 
	 -65,   44,  116,   91,  -11, 
	-104, -109,  -24,   82,  119, 
	  56,  -54, -119,  -84,   21, 
	 109,  105,   13,  -90, -117, 
	 -46,   63,  120,   76,  -32, 
	-113,  -99,   0, 
	};

const  char freq1336[] = { // 1333Hz 4cycle
	 104,  104,    1, -104, -104, 
	  -1,  104,  104,    1, -103, 
	-104,   -1,  103,  105,    1, 
	-103, -105,   -2,  103,  105, 
	   2, -103, -105,   0, 
	};

const  char freq1477[] = { // 1481Hz 5cycle
	 110,   87,  -41, -120,  -54, 
	  77,  115,   15, -103,  -97, 
	  26,  118,   67,  -65, -118, 
	 -29,   95,  105,  -12, -114, 
	 -79,   52,  120,   44,  -85, 
	-111,   0, 
	};

const  char	*PB_high[] = {
		freq1209, freq1336, freq1477,	// 1 2 3
		freq1209, freq1336, freq1477,	// 4 5 6
		freq1209, freq1336, freq1477,	// 7 8 9
		freq1209, freq1336, freq1477,	// * 0 #
	};	

const  char	*PB_low[] =  {
		freq697, freq697, freq697,	// 1 2 3
		freq770, freq770, freq770,	// 4 5 6
		freq852, freq852, freq852,	// 7 8 9
		freq941, freq941, freq941,	// * 0 #
	};	



/** Function Prototype ****************************************************/

#define	ON	1
#define	OFF	0

extern	int	dtmf_ctrl;

static	int	hfidx;
static	int	lfidx;
static	const char *highf;
static	const char *lowf;
static	int	outcount;

/*----------------------------------------------------------------------
	dtmf raw sampling
	This function called by T1interrupt routine.
----------------------------------------------------------------------*/

int		dtmf_raw(void)
{
	int high_val, low_val;

	// pickup next value
	high_val = highf[hfidx];
	low_val = lowf[lfidx];

	// update next index
	if(high_val == 0) hfidx = 0; else hfidx++;
	if(low_val == 0) lfidx = 0; else lfidx++;

	// increment outcount.
	// when outcount was reached 640count(means 80msec), output to stop automachicary.
	if(++outcount >= 640){
		dtmf_ctrl = OFF;
	}

	return (high_val + low_val); // max (120+120)/3=80
}


/*--------------------------------------------------------------------------
	dtmf tone
    dn: '0'-'9','*', '#' tone output start
		other output stop 
--------------------------------------------------------------------------*/

void	dtmf_tone(int dn)
{
	if(dn >= '1' && dn <= '9'){
		dn -= '1';
	}else if(dn == '0'){
		dn = 10;
	}else if(dn == '*'){
		dn = 9;
	}else if(dn == '#'){
		dn = 11;
	}else{
		dtmf_ctrl = OFF;
		return;
	}

	// DTMF generate
	asm("DISI #100");
	highf =  PB_high[dn];
	lowf = PB_low[dn];
	hfidx = lfidx = outcount = 0;
	asm("DISI #0");

	// tone output start
	dtmf_ctrl = ON;

}

/*** end of dtmf.c *********************************************************/
