/*************************************************************

	File: lcd.h

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

*/

#ifndef	__LCD_H
#define	__LCD_H

// AC is address counter
#define	cmd_CLEAR			0x01	// clear display
#define	cmd_HOME			0x02	// return home, reset DDRAM AC 00
#define	cmd_MODE_SET		0x04	// set the moving direction of cursor and display
	#define	prm_MODE_INC		0x02	// move to right DDRAM AC++
	#define	prm_MODE_DEC		0x00	// move to left DDRAM AC--
	#define	prm_MODE_SHIFT		0x01	// shift of entire display(MODE_INC=left,MODE_DEC=right)
	#define	prm_MODE_NON_SHIFT	0x00	// don't shift
#define	cmd_DISPLAY			0x08	// Control display/cursor/blink ON/OFF 1 but register
	#define	prm_DISPLAY_ON		0x04	// display is turned on
	#define	prm_DISPLAY_OFF		0x00	// display is turned off
	#define	prm_CURSOR_ON		0x02	// cursor is turned on
	#define	prm_CURSOR_OFF		0x00	// cursor is turned off
	#define	prm_BLINK_ON		0x01	// cursor blink is on
	#define	prm_BLINK_OFF		0x00	// cursor blink is off
#define	cmd_SHIFT			0x10	// Cursor or Display Shift
	#define	prm_DISPLAY_SHIFT	0x08	// select display shift(AC don't change)
	#define	prm_CURSOR_SHIFT	0x00	// select cursor shift(AC change ++/--)
	#define	prm_RIGHT_SHIFT		0x04	// shited to right
	#define	prm_LEFT_SHIFT		0x00	// shifted to left
#define	cmd_SET_FUNCTION	0x20	// Function Set
	#define	prm_8BIT			0x10	// MPU interface 8-bit operation
	#define	prm_4BIT			0x00	// MPU interface 4-bit operation
	#define	prm_2LINE			0x08	// 2-line display mode is set
	#define	prm_1LINE			0x00	// 1-line display mode is set
	#define	prm_LARGE_FONT		0x04	// 5x16 dot in 1-line mode, don't effect in 2-line mode
	#define	prm_SMALL_FONT		0x00	// 5x8 dot
	#define	prm_EXT_INSTRCT		0x01	// select extension instruction
	#define	prm_NORMAL_INSTRCT	0x00	// select normal instruction
#define	cmd_SET_CGADR		0x40	// set CG RAM ADDRESS, parameter is 0x00 - 0x3F
#define	cmd_SET_DDADR		0x80	// set DD RAM ADDRESS, parameter is 0x00-0x27,0x40-0x67

// extension instruction
#define	cmd_FOLLOWER		0x6a	// Follower control
	// follower circuit is turned on, V0 ampliffied ratio is 0b010
#define	cmd_SET_ICONADR		0x40	// set ICON RAM ADDRESS, parameter is 0x00 - 0x0f
#define	cmd_OSC_ADJ			0x10	// set OSC frequency adjust, parameter is 0x00 - 0x07, default is 0x04
#define	cmd_ICON_CONTRAST_H	0x54	// Power/ICON control/Contrast(high)
	#define	prm_ICON_ON		0x08	// ICON is turned on
	#define	prm_ICON_OFF	0x00	// ICON is turned off
	// parameter contrast high 2 bit, (contrast >> 4) & 0b11
#define	cmd_CONTRAST_L		0x70
	// parameter contrast low 4 bit, contrast & 0b1111


#define	LCD_DSP_LENGTH	16
#define	LCD_SET_LINE1	(cmd_SET_DDADR | 0x00)
#define	LCD_SET_LINE2	(cmd_SET_DDADR | 0x40)

#define	ON	1
#define	OFF	0

extern	void	lcd_initialize(void);
extern	void	icon_telephone(int sw);
extern	void	icon_ring_bell(int sw);
extern	void	lcd_cmdwrite(unsigned char cmd);
extern	void	lcd_datawrite(char data);
extern	void	lcd_strwrite(char *str);
extern	void	puticon(unsigned short flg);

extern	void	i2cOpenStart(void);
extern	void	i2cReStart(void);
extern	void	i2cStopClose(void);
extern	int		i2cSend(unsigned char data);
extern	unsigned char	i2cRec(void);
extern	void	i2cAckSend(void);
extern	void	i2cNAckSend(void);


#endif	// __LCD_H

/*** end of lcd.h ******************************************/
