/******************************************************************************

	File: message.c

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

	This file is part of IPphone.
	making the SIP request and response message 

******************************************************************************/

#include "typedef.h"
#include "pic_print.h"
#include "pic_string.h"
#include "pic_timer.h"
#include "pic_delay.h"
#include "digest.h"
#include "sysconf.h"
#include "message.h"
#include "ua.h"


static	short  	rndwrk = -1;
static	char	sdp[150];

/*---------------------------------------------------------------------------
	rand()
---------------------------------------------------------------------------*/

void	srand(int x)
{
	rndwrk = x;
}

int		rand(void)
{
	/* Liner congurential generator
	 *  A=512(4*128)+1=513(0x201)
	 *  B=59
	 *	M=32768(4*8192)(0x8000)
	 *  Xn = (A*Xn-1 +B)%M
	 */

	if(rndwrk == -1){	// Initial randam seed is timer clock value from Power On time.
		rndwrk = (timer_clock & 0x7fff);
	}

    rndwrk = (513L * (long)rndwrk + 59L) & 0x7fff;
	return rndwrk;
}


/*-----------------------------------------------------------------------------
	make command sequense
-----------------------------------------------------------------------------*/

int		make_seq(void)
{
	static	unsigned int seq_number;

	seq_number++;

	return (seq_number & 0x7fff);

}


/*-----------------------------------------------------------------------------
	make branch string
	buffer size must 18byte, buf[18]
-----------------------------------------------------------------------------*/

char	*make_branch(char *buf)
{
	mysprintf(buf, "z9hG4bK%04x%04x", rand(), rand());
	return buf;
}

/*-----------------------------------------------------------------------------
	make tag string
	buffer size must 10byte, buf[10]
-----------------------------------------------------------------------------*/

char	*make_tag(char *buf)
{
	mysprintf(buf, "%04x%04x", rand(), rand());
	return buf;
}

/*-----------------------------------------------------------------------------
	make call-id string
	buffer size must 32byte, buf[32]
-----------------------------------------------------------------------------*/

char	*make_call_id(char *buf)
{
	mysprintf(buf, "%04x%04x%02x@%s", rand(), rand(), rand(), get_local_sip_ip());
	return buf;
}


/*-----------------------------------------------------------------------------
	get_to_tag()
	search to_tag in rx_buffer
caution: this function destroyed message in rx_buf.  
-----------------------------------------------------------------------------*/

char	*get_to_tag(void)
{
	char	*p1, *p2, *p3;

	p1 = stristr(rx_buf, "To:");
	if(p1 == NULL) return NULL;	// not found To header
	p2 = strchr(p1, '\r'); *p2 = 0;
	p1 = stristr(p1, "tag=");
	if(p1 == NULL) return NULL;	// not found to_tag in To header
	p1 += 4;
	p3 = p1;
	while(*p3 != ';' && *p3 != '>' && *p3 != ' ' && *p3 == 0) p3++;
	*p3 = 0;

	return p1;
}


/*-----------------------------------------------------------------------------------------
	get remote_rtp_ip, remote_rtp_port from received sdp in rx_buf
-----------------------------------------------------------------------------------------*/

void	anlys_sdp(void)
{
	char	*sp, *ep;

	sp = stristr(rx_buf, "c=IN IP4 "); if(sp == NULL) return;
	sp += strlen("c=IN IP4 ");
	ep = sp;
	while(*ep  && *ep != '\r') ep++;
	*ep = 0; set_remote_rtp_ip(sp); *ep = '\r';

	sp = stristr(rx_buf, "m=audio "); if(sp == NULL) return; 
	sp += strlen("m=audio ");
	set_remote_rtp_port(atoi(sp));

}


/*-----------------------------------------------------------------------------
	header_copy()
------------------------------------------------------------------------------*/

char	*header_copy(char *orig, char *buf, char *hname)
{
	char	*s;

	s = stristr(orig, hname);
	if(s == NULL){
		buf[0] = 0;
		return buf;
	}

	while((*buf++ = *s++) != '\n');
	*buf = 0;

	return buf;
}

/*-----------------------------------------------------------------------------
	header_value_copy()
------------------------------------------------------------------------------*/

char	*header_value_copy(char *orig, char *buf, char *hname)
{
	char	*s;

	s = stristr(orig, hname);
	if(s == NULL){
		buf[0] = 0;
		return buf;
	}

	s = strchr(s, ':') +1;
	while((*buf++ = *s++) != '\n');
	*buf = 0;

	return buf;
}

/*-----------------------------------------------------------------------------
	write to buffer, authentication header value 
	makes from 401/407 sip message in rx_buf, 

	401 hedaer
	WWW-Authenticate: Digest realm="sip.suwa-koubou.jp",
		  domain="192.168.1.10",nonce="4ee5894f4ee58950",
		  opaque="",stale=FALSE,algorithm=MD5
------------------------------------------------------------------------------*/

char	*make_auth(char *buf, char *request)
{
	char	uri[24];
	char	*realm, *p1;
	char	*nonce, *p2;
	char	*res;

	realm = stristr(rx_buf, "realm=");
	nonce = stristr(rx_buf, "nonce=");
	if(realm == NULL || nonce == NULL) return NULL;

	realm += 7;
	p1 = realm;
	while(*p1 != '"') p1++;
	*p1 = 0;

	nonce += 7;
	p2 = nonce;
	while(*p2 != '"') p2++;
	*p2 = 0;	

	mysprintf(uri, "sip:%s", get_remote_sip_ip());
	res = digest_calc_response(get_sip_id(), realm, get_sip_pswrd(), request, uri, nonce); 

	mysprintf(buf, "Digest username=\"%s\",realm=\"%s\",nonce=\"%s\",opaque=\"\",algorithm=MD5,uri=\"%s\",response=\"%s\"",
		get_sip_id(), realm, nonce, uri, res);
 
	*p1 = *p2 = '"';

	return buf;
}

/*------------------------------------------------------------------------------
	make_REGISTER_msg()
------------------------------------------------------------------------------*/

char *make_REGISTER_msg(char *branch, char *from_tag, char *to_tag, char *call_id,
		int cmdseq, char *auth)
{
	msg_buf[0] = 0;
	mysprintf(msg_buf+strlen(msg_buf), "REGISTER sip:%s SIP/2.0\r\n", get_remote_sip_ip());
	mysprintf(msg_buf+strlen(msg_buf), "Via: SIP/2.0/UDP %s:%u;branch=%s\r\n", get_local_sip_ip(), get_local_sip_port(), branch);
	mysprintf(msg_buf+strlen(msg_buf), "From: <sip:%s@%s>;tag=%s\r\n", get_tel_num(), get_remote_sip_ip(), from_tag);
	if(to_tag != NULL){
		mysprintf(msg_buf+strlen(msg_buf), "To: <sip:%s@%s>;tag=%s\r\n", get_tel_num(), get_remote_sip_ip(), to_tag);
	}else{
		mysprintf(msg_buf+strlen(msg_buf), "To: <sip:%s@%s>\r\n", get_tel_num(), get_remote_sip_ip());
	}
	mysprintf(msg_buf+strlen(msg_buf), "Call-ID: %s\r\n", call_id);
	mysprintf(msg_buf+strlen(msg_buf), "CSeq: %d REGISTER\r\n", cmdseq);
	if(auth != NULL){
		mysprintf(msg_buf+strlen(msg_buf), "Authorization: %s\r\n", auth);
	}
	mysprintf(msg_buf+strlen(msg_buf), "Contact: <sip:%s@%s:%u>\r\n", get_tel_num(), get_local_sip_ip(), get_local_sip_port());
	mysprintf(msg_buf+strlen(msg_buf), "Expires: %d\r\nMax-Forwards: 70\r\nContent-Length: 0\r\n\r\n", get_expires_time());
	//mysprintf(msg_buf+strlen(msg_buf), "Expires: %d\r\nMax-Forwards: 70\r\nContent-Length: 0\r\n\r\n", EXPIRES_TIME);

	return msg_buf;
}


/*------------------------------------------------------------------------------
	make_INVITE_msg()
------------------------------------------------------------------------------*/

char *make_INVITE_msg(char *tel, char *branch, char *from_tag,
		 char *to_tag, char *call_id, int cmdseq, char *auth)
{
	return make_request_msg("INVITE", tel, branch, call_id, from_tag, to_tag, cmdseq, cmdseq, auth, CALLER);
}


/*------------------------------------------------------------------------------
	make sdp
------------------------------------------------------------------------------*/

char *make_sdp(void)
{
	mysprintf(sdp, 
		"v=0\r\n"
		"o=- 0 0 IN IP4 %s\r\n"
		"s=-\r\n"
		"c=IN IP4 %s\r\n"
		"t=0 0\r\n"
		"m=audio %d RTP/AVP 0\r\n"
		"a=rtpmap:0 PCMU/8000\r\n"
		"a=ptime:20\r\n",
		get_local_rtp_ip(), get_local_rtp_ip(), get_local_rtp_port());
	return sdp;
}


/*------------------------------------------------------------------------------
	make_ACK_msg()
	header copy from rx_buf to msg_buf
------------------------------------------------------------------------------*/

char *make_ACK_msg(char *tel, char *branch, int cmdseq)
{
	return make_request_msg("ACK", tel, branch, NULL, NULL, NULL, cmdseq, 0, NULL, CALLER);
}

/*------------------------------------------------------------------------------
	make_PRACK_msg()
	header copy from rx_buf to msg_buf
------------------------------------------------------------------------------*/

char *make_PRACK_msg(char *tel, char *branch, int cmdseq, int invite_cmdseq)
{
	return make_request_msg("PRACK", tel, branch, NULL, NULL, NULL, cmdseq, invite_cmdseq, NULL, CALLER);
}


/*------------------------------------------------------------------------------
	make_CANCEL_msg()
------------------------------------------------------------------------------*/

char *make_CANCEL_msg(char *tel, char *branch, int cmdseq)
{
	return make_request_msg("CANCEL", tel, branch, NULL, NULL, NULL, cmdseq, 0, NULL, CALLER);
}


/*------------------------------------------------------------------------------
	make_BYE_msg()
------------------------------------------------------------------------------*/

char *make_BYE_msg(char *tel, char *branch, int cmdseq, int call_type)
{
	return make_request_msg("BYE", tel, branch, NULL, NULL, NULL, cmdseq, 0, NULL, call_type);
}


/*------------------------------------------------------------------------------
	make_UPDATE_msg()
------------------------------------------------------------------------------*/

char *make_UPDATE_msg(char *tel, char *branch, int cmdseq, int call_type)
{
	return make_request_msg("UPDATE", tel, branch, NULL, NULL, NULL, cmdseq, 0, NULL, call_type);
}


/*------------------------------------------------------------------------------
	make_request_msg()
	INVITE/CANCEL/ACK/PRACK/BYE/UPDATE
------------------------------------------------------------------------------*/

char	*make_request_msg(char *method, char *tel, char *branch, char *call_id, 
						char *from_tag, char *to_tag, int cmdseq, int invite_cmdseq,
						char *auth, int call_type)
{
	char	*d, *p;
	int		rseq, flg_184;

	msg_buf[0] = 0;
	flg_184 = 0;

	// *** Request Line **************************
	if(strcmp(method, "INVITE") == 0){
		// at INVITE
		if(tel == NULL) return msg_buf;
		if(strncmp(tel, "184", 3) == 0){
			flg_184 = 1;
			strcpy(tel, tel+3);
		}else if(strncmp(tel, "186", 3) == 0){
			strcpy(tel, tel+3);
		}
		mysprintf(msg_buf+strlen(msg_buf), "INVITE sip:%s@%s SIP/2.0\r\n", tel, get_remote_sip_ip());
	}else{
		// at ACK/PRACK/CANCEL/BYE/UPDATE -> remote contact
		p = stristr(rx_buf, "Contact:");
		if(p != NULL){ // found Contact header
			mysprintf(msg_buf+strlen(msg_buf), "%s ", method);
			p = strchr(p, '<'); p++;
			d = msg_buf+strlen(msg_buf);
			while((*d++ = *p++) != '>');
			d--; *d = 0;
			mysprintf(msg_buf+strlen(msg_buf), " SIP/2.0\r\n");
		}else{	// when not found Conatct header, request line is remote telephone number instead contact value
			if(tel == NULL) return msg_buf;
			mysprintf(msg_buf+strlen(msg_buf), "%s sip:%s@%s SIP/2.0\r\n", method, tel, get_remote_sip_ip());
		}
	}

	// *** Headers *********************************
	//  Via
	if(branch != NULL){
		mysprintf(msg_buf+strlen(msg_buf), "Via: SIP/2.0/UDP %s:%u;branch=%s\r\n", get_local_sip_ip(), get_local_sip_port(), branch);
	}else{
		// branch of ACK is same INVITE via branch.
		header_copy(rx_buf, msg_buf+strlen(msg_buf), "Via:");
	}

	//  From, To, Call-ID
	if(strcmp(method, "INVITE") == 0){
		// at INVITE
		if(!flg_184){
			mysprintf(msg_buf+strlen(msg_buf), "From: <sip:%s@%s>;tag=%s\r\n", get_tel_num(), get_remote_sip_ip(), from_tag);
		}else{
			mysprintf(msg_buf+strlen(msg_buf), "From: \"Anonymous\" <sip:anonymous@anonymous.invalid>;tag=%s\r\n", from_tag);
			mysprintf(msg_buf+strlen(msg_buf), "Privacy: id\r\n");
			mysprintf(msg_buf+strlen(msg_buf), "P-Preferred-identity: <sip:%s@%s>\r\n", get_tel_num(), get_remote_sip_ip());
		}
		if(to_tag != NULL){
			mysprintf(msg_buf+strlen(msg_buf), "To: <sip:%s@%s>;tag=%s\r\n", tel, get_remote_sip_ip(), to_tag);
		}else{
			mysprintf(msg_buf+strlen(msg_buf), "To: <sip:%s@%s>\r\n", tel, get_remote_sip_ip());
		}
		mysprintf(msg_buf+strlen(msg_buf), "Call-ID: %s\r\n", call_id);

	}else if(call_type == CALLEE && (strcmp(method, "BYE") == 0 || strcmp(method, "UPDATE") == 0)){
		// at BYE/UPDATE if call_type is CALLEE
		strcpy(msg_buf+strlen(msg_buf), "From:");
		header_value_copy(rx_buf, msg_buf+strlen(msg_buf), "To:");
		strcpy(msg_buf+strlen(msg_buf), "To:");
		header_value_copy(rx_buf, msg_buf+strlen(msg_buf), "From:");
		header_copy(rx_buf, msg_buf+strlen(msg_buf), "Call-ID:");
	}else{
		// at ACK/CANCEL/PRACK/BYE/UPDATE
		header_copy(rx_buf, msg_buf+strlen(msg_buf), "From:");
		header_copy(rx_buf, msg_buf+strlen(msg_buf), "To:");
		header_copy(rx_buf, msg_buf+strlen(msg_buf), "Call-ID:");
	}

	// CSeq, Contact, Allow, MAX-Forwards, Content-Length
	mysprintf(msg_buf+strlen(msg_buf), "CSeq: %d %s\r\n", cmdseq, method);

	if(strcmp(method, "PRACK") == 0){
		// get Rseq
		p = stristr(rx_buf, "RSeq:"); if(p == NULL) return  msg_buf;
		p += 5;	// strlen("RSeq:") = 5
		rseq = atoi(p);
		mysprintf(msg_buf+strlen(msg_buf), "RAck: %d %d INVITE\r\n", rseq, invite_cmdseq); 
	}
	mysprintf(msg_buf+strlen(msg_buf), "Contact: <sip:%s:%u>\r\n", get_local_sip_ip(), get_local_sip_port());
	mysprintf(msg_buf+strlen(msg_buf), "Allow: INVITE,CANCEL,ACK,BYE,PRACK,UPDATE\r\n");
	mysprintf(msg_buf+strlen(msg_buf), "Max-Forwards: 70\r\n");

	if(strcmp(method, "INVITE") == 0){
		//mysprintf(msg_buf+strlen(msg_buf), "Supported: 100rel, timer\r\nSession-Expires: %d;refresher=uas\r\n", SESSION_TIME);
		mysprintf(msg_buf+strlen(msg_buf), "Supported: 100rel, timer\r\nSession-Expires: %d\r\n", get_session_time());
		if(auth != NULL) mysprintf(msg_buf+strlen(msg_buf), "Proxy-Authorization: %s\r\n", auth);
		make_sdp();
		mysprintf(msg_buf+strlen(msg_buf), "Content-Type: application/sdp\r\nContent-Length: %d\r\n\r\n", strlen(sdp));
		mysprintf(msg_buf+strlen(msg_buf), "%s", sdp);
	}else if(strcmp(method, "UPDATE") == 0){
		mysprintf(msg_buf+strlen(msg_buf), "Supported: timer\r\nSession-Expires: %d;refresher=uac\r\n", get_session_time());
		//mysprintf(msg_buf+strlen(msg_buf), "Supported: timer\r\nSession-Expires: %d;refresher=uac\r\n", SESSION_TIME);
		mysprintf(msg_buf+strlen(msg_buf), "Content-Length: 0\r\n\r\n");
	}else{	
		mysprintf(msg_buf+strlen(msg_buf), "Content-Length: 0\r\n\r\n");
	}	

	return msg_buf;
}


/*------------------------------------------------------------------------------
	make_response_msg()
	100,180,200,486,487,2000(INVITE response)
	header copy from rx_buf to msg_buf
------------------------------------------------------------------------------*/

static	int	timer_opt, refresher;

char	*make_response_msg(int status_code)
{
	char	to_tag[TAG_SIZE];
	char	*phrase, *p;

	msg_buf[0] = 0;

	// status line
	switch(status_code){
	case 100: phrase = "100 Trying"; break;
	case 180: phrase = "180 Ringing"; break;
	case 200: case 2000:
			  phrase = "200 Ok"; break;
	case 420: phrase = "420 Bad Extension"; break;
	case 422: phrase = "422 Session Interval Too Small"; break;
	case 486: phrase = "486 Busy Here"; break;
	case 487: phrase = "487 Request Terminated"; break;
	default: return msg_buf;
	}
	mysprintf(msg_buf, "SIP/2.0 %s\r\n", phrase);

	// Header Via
	p = rx_buf;
	while((p = stristr(p, "Via:")) != NULL){
		header_copy(p, msg_buf+strlen(msg_buf), "Via:");
		p += 5;
	}

	// Header From, To
	header_copy(rx_buf, msg_buf+strlen(msg_buf), "From:");
	header_copy(rx_buf, msg_buf+strlen(msg_buf), "To:");

	// 180 add to_tag
	if(status_code == 180){
		make_tag(to_tag);
		mysprintf(msg_buf + strlen(msg_buf) -2, ";tag=%s\r\n", to_tag);
	}

	// Call-ID, CSeq
	header_copy(rx_buf, msg_buf+strlen(msg_buf), "Call-ID:");
	header_copy(rx_buf, msg_buf+strlen(msg_buf), "CSeq:");

	// Allow, Contact
	if(status_code != 100){
		mysprintf(msg_buf+strlen(msg_buf), "Allow: INVITE,CANCEL,ACK,BYE,PRACK,UPDATE\r\n");
		mysprintf(msg_buf+strlen(msg_buf), "Contact: <sip:%s:%u>\r\n", get_local_sip_ip(), get_local_sip_port());
	}

	// Unsupported
	if(status_code == 420){
		strcpy(msg_buf+strlen(msg_buf), "Unsupported:");
		header_value_copy(rx_buf, msg_buf+strlen(msg_buf), "Require:");
	}

	// Min-SE
	if(status_code == 422){
		mysprintf(msg_buf+strlen(msg_buf), "Min-SE: %d\r\n", get_session_time());
	}

	if(status_code == 2000){
		// Require, Content-Type, Content-Length, sdp
		make_sdp();
		if(timer_opt){
			mysprintf(msg_buf+strlen(msg_buf), "Require: timer\r\nSession-Expires: %d;refresher=", get_session_time());
			//mysprintf(msg_buf+strlen(msg_buf), "Require: timer\r\nSession-Expires: %d;refresher=", SESSION_TIME);
			if(refresher){
				mysprintf(msg_buf+strlen(msg_buf), "uas\r\n");
			}else{
				mysprintf(msg_buf+strlen(msg_buf), "uac\r\n");
			}
		}
		mysprintf(msg_buf+strlen(msg_buf), "Content-Type: application/sdp\r\nContent-Length: %d\r\n\r\n", strlen(sdp));
		mysprintf(msg_buf+strlen(msg_buf), "%s", sdp);

	}else{
		if(status_code == 200 && stristr(rx_buf, "UPDATE") == rx_buf){
			mysprintf(msg_buf+strlen(msg_buf), "Require: timer\r\nSession-Expires: %d;refresher=uac\r\n", get_session_time());
			//mysprintf(msg_buf+strlen(msg_buf), "Require: timer\r\nSession-Expires: %d;refresher=uac\r\n", SESSION_TIME);
		}
		// Content-Length
		mysprintf(msg_buf+strlen(msg_buf), "Content-Length: 0\r\n");
		mysprintf(msg_buf+strlen(msg_buf), "\r\n");
	}

	return msg_buf;
}

/*------------------------------------------------------------------------------
	make_100_msg()
	header copy from rx_buf to msg_buf
------------------------------------------------------------------------------*/

char *make_100_msg(void)
{
	return make_response_msg(100);
}


/*------------------------------------------------------------------------------
	make_180_msg()
	header copy from rx_buf to msg_buf
------------------------------------------------------------------------------*/

char *make_180_msg(void)
{
	return make_response_msg(180);
}


/*------------------------------------------------------------------------------
	make_200OK_msg()	for received INVITE 
	header copy from saved_msg to msg_buf
------------------------------------------------------------------------------*/

char	*make_200OK_msg(int tmopt, int sender)
{
	timer_opt = tmopt;
	refresher = sender;
	return make_response_msg(2000);
}


/*------------------------------------------------------------------------------
	make_200_msg() for received CANCEL/BYE/UPDATE
------------------------------------------------------------------------------*/

char *make_200_msg(void)
{
	return make_response_msg(200);
}


/*------------------------------------------------------------------------------
	make_420_msg()
------------------------------------------------------------------------------*/

char *make_420_msg(void)
{
	return make_response_msg(420);
}


/*------------------------------------------------------------------------------
	make_422_msg()
------------------------------------------------------------------------------*/

char *make_422_msg(void)
{
	return make_response_msg(422);
}


/*------------------------------------------------------------------------------
	make_486_msg()
------------------------------------------------------------------------------*/

char *make_486_msg(void)
{
	return make_response_msg(486);
}


/*------------------------------------------------------------------------------
	make_487_msg()
------------------------------------------------------------------------------*/

char *make_487_msg(void)
{
	return make_response_msg(487);
}


/*** end of message.c *****************************************************************/
