/***************************************************************************

	File: pic_232.c


	Copyright (C) 2010-2012 S.Suwa <www.suwa-koubou.jp>

	RS232C handling routine  for PIC24FJ64GA002

	PC to connect by RS232C
		RX		 U1RX(RP11,RB11)
		TX		 U1TX(RP10,RB10)
		Fcy 	 16MHz
		baudrate 19200
*/

#include <p24FJ64GA002.h>
#include "pic_delay.h"
#include "pic_232.h"


#define	RS_TIME_OUT		1000

static int EchoBack;	// receive byte echo send back switch.


/*--------------------------------------------------------------------------
	rs232c initialize
--------------------------------------------------------------------------*/

void rs_initialize(void)
{
	// set U1 RS232
	U1MODE = 0x8000;	// N81 
	U1BRG = 51;			// 19200bps(16MHz/16/19200-1=51)
	U1STA = 0x0400;		// start transmitssion UTXEN=1

	EchoBack = 1;		// set echo on
}


/*--------------------------------------------------------------------------
	rs_set_echo(int echo_sw)
		EchoBack = ECHO_ON  rs_getchr() -> rs_putchr()
--------------------------------------------------------------------------*/

void rs_set_echo(int echo_sw)
{
	EchoBack = echo_sw;
}


/*--------------------------------------------------------------------------
	rs_getstr(char *buff) end_charactor is NULL char('\0')
--------------------------------------------------------------------------*/

char *rs_getstr(char *buff)
{
	int	c;
	char *p;

	p = buff;

	for(;;){
		while ((c = rs_getchr()) == -1);
		if(c == '\r') break;
		*p++ = c;
	}

	*p = '\0';	// set EOS
	return buff;
}



/*--------------------------------------------------------------------------
	rs_getchr()  none delay, without timeout
--------------------------------------------------------------------------*/

int rs_getchr(void)
{
	int	c;

	if (U1STAbits.URXDA == 0) return -1;

	c = U1RXREG;

	if(EchoBack){
		while(rs_putchr(c) == -1);
		if(c == '\r'){
			while(rs_putchr('\n') == -1);
		}
	}
	return c;
}


/*--------------------------------------------------------------------------
	rs_getwchr()  wait for RXRDY, with timeout
--------------------------------------------------------------------------*/

int rs_getwchr(void)
{
	int	i, c;

	for(i = 0; i < RS_TIME_OUT; i++){
		if (U1STAbits.URXDA == 1){
			c = U1RXREG;
			if(EchoBack){
				while(rs_putchr(c) == -1);
				if(c == '\r'){
					while(rs_putchr('\n') == -1);
				}
			}
			return c;
		}
		delay_ms(1);
	}
	return -1;
}


/*--------------------------------------------------------------------------
	rs_putstr(char *buff)
--------------------------------------------------------------------------*/

void rs_putstr(char *buff)
{
	while(*buff){
		rs_putwchr(*buff);
		buff++;
	}
}


/*--------------------------------------------------------------------------
	rs_putchr(int c)
--------------------------------------------------------------------------*/

int rs_putchr(int c)
{
	if (U1STAbits.UTXBF == 1) return -1;
	U1TXREG = c & 0xff;
	return c;
}


/*--------------------------------------------------------------------------
	rs_putwchr(int c)
--------------------------------------------------------------------------*/

int rs_putwchr(int c)
{
	while(rs_putchr(c) == -1);
	return c;
}

/*** end of pic_232.c ******************************************************/
