/******************************************************************

	File: pic_print.c

	Copyright (C) 2010-2012 S.Suwa <www.suwa-koubou.jp>

	convert binary to decimal/hex/ascii, 
		write to memory buffer

*/

/** INCLUDES **************************************************************/
#include "pic_print.h"
#include "pic_232.h"


/** DEFINITION ************************************************************/ 
#define	NULL	0

/** VARIABLES *************************************************************/ 
static char hex_str[] = "0123456789abcdef";
static char pbuff[16];	// for myprintf


static	int	intval(char *p)
{
	int	hd, ld;

	ld = (*p++) & 0xff;
	hd = (*p) & 0xff;
	return (hd << 8) + ld;

}

static	long longval(char *p)
{
	long	hhd, hld, lhd, lld;

	lld = (*p++) & 0xff;
	lhd = (*p++) & 0xff;
	hld = (*p++) & 0xff;
	hhd = (*p++) & 0xff;

	return (hhd << 24) + (hld << 16) + (lhd << 8) + lld;
}


/*==========================================================================
	myprintf()	printout to rs_232c 
==========================================================================*/

void	myprintf(char *format, ...)
{
	char *vap;
	char *p, *sp;
	int	width;
	int	lz_flg;	// leading "0"
	int	long_flg;
	int	len;

	p = format;
	vap = (char *)&format; 

	while(*p){
		switch(*p){
		case '%':
			width = 0;
			lz_flg = 0;
			long_flg = 0;

			for(;;){
				p++;

				if(*p == 'd'){
					if(long_flg){
						vap -= sizeof(long);				
						//rs_putstr(long_to_decimal(pbuff, *(long *)vap));
						rs_putstr(long_to_decimal(pbuff, longval(vap)));
					}else if(width && lz_flg){
						vap -= sizeof(int);
						//rs_putstr(int_to_0ndecimal(pbuff, width, *(int *)vap));
						rs_putstr(int_to_0ndecimal(pbuff, width, intval(vap)));
					}else if(width && ! lz_flg){
						vap -= sizeof(int);
						//rs_putstr(int_to_ndecimal(pbuff, width, *(int *)vap));
						rs_putstr(int_to_ndecimal(pbuff, width, intval(vap)));
					}else{
						vap -= sizeof(int);
						//rs_putstr(int_to_decimal(pbuff, *(int *)vap));
						rs_putstr(int_to_decimal(pbuff, intval(vap)));
					}
					break;
				}

				if(*p == 'u'){
					if(long_flg){				
						vap -= sizeof(long);				
						//rs_putstr(ulong_to_decimal(pbuff, *(unsigned long *)vap));
						rs_putstr(ulong_to_decimal(pbuff, (unsigned long)longval(vap)));
					}else if(width && lz_flg){
						vap -= sizeof(int);
						//rs_putstr(uint_to_0ndecimal(pbuff, width, *(unsigned int *)vap));
						rs_putstr(uint_to_0ndecimal(pbuff, width, (unsigned)intval(vap)));
					}else if(width && ! lz_flg){
						vap -= sizeof(int);
						//rs_putstr(uint_to_ndecimal(pbuff, width, *(unsigned int *)vap));
						rs_putstr(uint_to_ndecimal(pbuff, width, (unsigned)intval(vap)));
					}else{
						vap -= sizeof(int);
						//rs_putstr(uint_to_decimal(pbuff, *(unsigned int *)vap));
						rs_putstr(uint_to_decimal(pbuff, (unsigned)intval(vap)));
					}
					break;
				}

				if(*p == 'X' || *p == 'x'){
					if(! width && ! lz_flg){
						vap -= sizeof(int);
						//rs_putstr(int_to_hex(pbuff, *(unsigned int *)vap));
						rs_putstr(int_to_hex(pbuff, (unsigned)intval(vap)));
					}else if(width == 2){
						vap -= sizeof(int);
						//rs_putstr(int_to_2hex(pbuff, *(unsigned int *)vap));
						rs_putstr(int_to_2hex(pbuff, (unsigned)intval(vap)));
					}else{
						vap -= sizeof(int);
						//rs_putstr(int_to_4hex(pbuff, *(unsigned int *)vap));
						rs_putstr(int_to_4hex(pbuff, (unsigned)intval(vap)));
					}
					break;
				}

				if(*p == 'c'){	// ignore width, lead zero, long flg
					vap -= sizeof(int);
					//rs_putwchr( (*(int *)vap) & 0xff );
					rs_putwchr( intval(vap) & 0xff );
					break;
				}

				if(*p == 's'){
					vap -= sizeof(char *);
					//sp = *(char **)vap;
					sp = (char *)intval(vap);
					len = 0;

					if(width){
						while(len < width){
							if(*sp == 0){
								rs_putwchr(' ');
							}else{
								rs_putwchr(*sp++);
							}
							len++;
						}
					}else{
						while(*sp != 0){
							rs_putwchr(*sp++);
						}
					}
					break;
				}

				if(*p == '0' && ! width && ! lz_flg ){
					lz_flg = 1;

				}else if(*p >= '0' && *p <= '9'){
					width *= 10;
					width += *p - '0';

				}else if(*p == 'l' && ! long_flg){
					long_flg = 1;

				}else{
					break;
				}
			} // end for
			break;

		default:
			rs_putwchr(*p);
			break;

		} // end switch

		p++;

	} // end while, reached format string end

}


/*==========================================================================
	mysprintf()	printout to string buffer 
==========================================================================*/

void	mysprintf(char *strbuf, char *format, ...)
{
	char *vap;
	char *s, *p, *sp;
	int	width;
	int	lz_flg;	// leading "0"
	int	long_flg;

	s = strbuf;
	p = format;
	vap = (char *)&format; 

	while(*p){
		switch(*p){
		case '%':
			width = 0;
			lz_flg = 0;
			long_flg = 0;

			for(;;){
				p++;

				if(*p == 'd'){
					if(long_flg){
						vap -= sizeof(long);				
						//long_to_decimal(s, *(long *)vap);
						long_to_decimal(s, longval(vap));
					}else if(width && lz_flg){
						vap -= sizeof(int);
						//int_to_0ndecimal(s, width, *(int *)vap);
						int_to_0ndecimal(s, width, intval(vap));
					}else if(width && ! lz_flg){
						vap -= sizeof(int);
						//int_to_ndecimal(s, width, *(int *)vap);
						int_to_ndecimal(s, width, intval(vap));
					}else{
						vap -= sizeof(int);
						//int_to_decimal(s, *(int *)vap);
						int_to_decimal(s, intval(vap));
					}
					break;
				}

				if(*p == 'u'){
					if(long_flg){				
						vap -= sizeof(long);				
						//ulong_to_decimal(s, *(unsigned long *)vap);
						ulong_to_decimal(s, (unsigned long)longval(vap));
					}else if(width && lz_flg){
						vap -= sizeof(int);
						//uint_to_0ndecimal(s, width, *(unsigned int *)vap);
						uint_to_0ndecimal(s, width, (unsigned)intval(vap));
					}else if(width && ! lz_flg){
						vap -= sizeof(int);
						//uint_to_ndecimal(s, width, *(unsigned int *)vap);
						uint_to_ndecimal(s, width, (unsigned)intval(vap));
					}else{
						vap -= sizeof(int);
						//uint_to_decimal(s, *(unsigned int *)vap);
						uint_to_decimal(s, (unsigned)intval(vap));
					}
					break;
				}

				if(*p == 'X' || *p == 'x'){
					if(! width && ! lz_flg){
						vap -= sizeof(int);
						//int_to_hex(s, *(unsigned int *)vap);
						int_to_hex(s,  (unsigned)intval(vap));
					}else if(width == 2){
						vap -= sizeof(int);
						//int_to_2hex(s, *(unsigned int *)vap);
						int_to_2hex(s,  (unsigned)intval(vap));
					}else{
						vap -= sizeof(int);
						//int_to_4hex(s, *(unsigned int *)vap);
						int_to_4hex(s,  (unsigned)intval(vap));
					}
					break;
				}

				if(*p == 'c'){	// ignore width, lead zero, long flg
					vap -= sizeof(int);
					//*s = (*(int *)vap) & 0xff;
					*s = intval(vap) & 0xff;
					*(s +1) = 0;
					break;
				}

				if(*p == 's'){
					vap -= sizeof(char *);
					//sp = *(char **)vap;
					sp = (char *)intval(vap);
					{
						char	*dst;
						int		len;
	
						dst = s;
						len = 0;

						if(width){
							while(len < width){
								if(*sp == 0){
									*dst++ = ' ';
								}else{
									*dst++ = *sp++;
								}
								len++;
							}
							*dst = 0;
						}else{
							while((*dst++ = *sp++) != 0);
						}
					}
					break;
				}

				if(*p == '0' && ! width && ! lz_flg ){
					lz_flg = 1;

				}else if(*p >= '0' && *p <= '9'){
					width *= 10;
					width += *p - '0';

				}else if(*p == 'l' && ! long_flg){
					long_flg = 1;

				}else{
					*s = 0;
					break;
				}
			} // end for

			while(*s) s++;	// find terminate
							// int_to_decimal() Ȃǂ s ɏ񂾕񒷂͕sȂ̂ŁA
							// 񂾕̏I[܂ s CNgB
			break;

		default:
			*s++ = *p;
		} // end switch

		p++;

	} // end while, reached format string end

	*s = 0;

}



/*--------------------------------------------------------------------------
	format convert
--------------------------------------------------------------------------*/

// %X 
char *int_to_hex(char *buff, unsigned int d)
{
	char *p, *bp;

	int_to_4hex(buff, d);
	bp = p = buff;
	while(*p == '0') p++;	// zero cut
	while(*p){
		*bp++ = *p++;
	}
	return buff;
}


// %02X 
char *int_to_2hex(char *buff, unsigned int d)
{
	buff[0] = hex_str[(d & 0xf0) >> 4];
	buff[1] = hex_str[ d & 0xf ];
	buff[2] = 0;
	return buff;
}


// %04X 
char *int_to_4hex(char *buff, unsigned int d)
{
	int_to_2hex(buff, d >> 8);
	int_to_2hex(buff +2, d);
	return buff;
}


// %u  őTōlóiȂj
// 0 - 65535
char *uint_to_decimal(char *buff, unsigned int d)
{
	unsigned int div, base, i;

	for(i = 0, base = 10000; base > 0; base /= 10){
		div = d / base;
		d = d % base;
		if(div == 0 && i == 0 && base != 1) continue;
		buff[i++] = '0' + div;
	}
	buff[i] = 0;
	return buff;
}


// %d  őUōlóitj
// -32768 - 32767
char *int_to_decimal(char *buff, int d)
{
	if(d < 0){
		buff[0] = '-';
		uint_to_decimal(buff+1, -d);
	}else{
		uint_to_decimal(buff, (unsigned)d);
	}

	return buff;
}


// %0nu  ő傎ElóA擪͂O
char *uint_to_0ndecimal(char *buff, int n, unsigned int d)
{
	int	i;

	buff[n] = 0;
	for(i = n; i > 0; i--){
		buff[i-1] = '0' + d % 10;
		d /= 10;
	}
	return buff;
}


// %0nd  ő傎ElóA擪͂O
char *int_to_0ndecimal(char *buff, int n, int d)
{
	char sgn;

	if(d < 0){
		sgn = '-';
		d = -d;
	}else{
		sgn = '0';
	}
	uint_to_0ndecimal(buff, n, d);
	if(sgn == '-') buff[0] = sgn;
	return buff;
}


// %nu  ő傎ElóA擪͋󔒖
char *uint_to_ndecimal(char *buff, int n, unsigned int d)
{
	int	i, mod;

	buff[n] = 0;
	for(i = n; i > 0; i--){
		mod = d % 10;
		d /= 10;
		if(mod == 0 && d == 0 && i != n){
			buff[i-1] = ' ';
		}else{
			buff[i-1] = '0' + mod;
		}
	}
	return buff;
}


// %nd  ő傎ElóA擪͋󔒖
char *int_to_ndecimal(char *buff, int n, int d)
{
	char sgn;
	int	i, mod, spos;

	buff[n] = 0;

	if(d < 0){
		sgn = '-';
		d = -d;
	}else{
		sgn = ' ';
	}

	for(spos = -1, i = n; i > 0; i--){
		mod = d % 10;
		d /= 10;
		if(mod == 0 && d == 0 && i != n){
			if(spos == -1) spos = i;
			buff[i-1] = ' ';
		}else{
			buff[i-1] = '0' + mod;
		}
	}
	if(spos != -1) buff[spos -1] = sgn;
	return buff;
}


// %lu  ő10ōlóiȂj
// 0 - 4294967295
char *ulong_to_decimal(char *buff, unsigned long d)
{
	unsigned long div, base;
	unsigned int	 i;

	for(i = 0, base = 1000000000L; base > 0; base /= 10L){
		div = d / base;
		d = d % base;
		if(div == 0 && i == 0 && base != 1) continue;
		buff[i++] = '0' + div;
	}
	buff[i] = 0;
	return buff;
}

// %ld  ő10ōlóiȂj
// -2147483648 - 2147483647
char *long_to_decimal(char *buff, long d)
{
	if(d < 0L){
		buff[0] = '-';
		ulong_to_decimal(buff+1, -d);
	}else{
		ulong_to_decimal(buff, (unsigned long)d);
	}
	return buff;
}


/*==========================================================================
	string convert basic function
===========================================================================*/

/*--------------------------------------------------------------------------
	ascii to integer
--------------------------------------------------------------------------*/

int	atoi(char *s)
{
	int	d;

	while(*s == ' ' || *s == '\t') s++;

	d = 0;
	while(*s >= '0' && *s <= '9'){
		d = d * 10 + *s - '0';
		s++;
	}
	return d;
}


/*--------------------------------------------------------------------------
	hex ascii to integer
--------------------------------------------------------------------------*/

int	htoi(char *s)
{
	int	d, x;

	while(*s == ' ' || *s == '\t') s++;

	d = 0;
	for(;;){
		if(*s >= '0' && *s <= '9'){
			x = *s - '0';
		}else if(*s >= 'A' && *s <= 'F'){
			x = *s - 'A' + 10;
		}else if(*s >= 'a' && *s <= 'f'){
			x = *s -'a' + 10;
		}else{
			break;
		}
		d = d * 16 + x;
		s++;
	}

	return d;

}	


/*** end of pic_print.c ********************************************************/
