/**************************************************

	File: pic_string.c

	Copyright (C) 2011-2012 S.Suwa <www.suwa-koubou.jp>

*/

#include "pic_string.h"

/*--------------------------------------------------
	 memory_copy() copy memory to memory.
--------------------------------------------------*/

void	memory_copy(uchar *dst, uchar *src, uint size)
{
	int	n;
	for(n = 0; n < size; n++){
		*dst++ = *src++;
	}
}


/*--------------------------------------------------
	 memory_set() set to memory.
--------------------------------------------------*/

void	memory_set(uchar *buf, uchar c, uint size)
{
	int	n;
	for(n = 0; n < size; n++){
		*buf++ = c;
	}
}


/*--------------------------------------------------
	 memory_comp() compare memory.
--------------------------------------------------*/

int		memory_comp(uchar *s1, uchar *s2, uint size)
{
	int	n;
	for(n = 0; n < size; n++, s1++, s2++){
		if(*s1 == *s2) continue; 
		if(*s1 < *s2) return -1;
		else return 1;
	}
	return 0;
}


/*---------------------------------------------------
	 strlen() lengh of string
---------------------------------------------------*/

uint	strlen(char *str)
{
	uint len;

	len = 0;
	while(*str++){
		len++;
	}
	return len;
}


/*--------------------------------------------------
	 strcmp() compare strings
--------------------------------------------------*/

int	strcmp(char *s1, char *s2)
{
	while(*s1){
		if(*s2 == 0) return 1;
		if(*s1 == *s2){
			s1++, s2++;
		}else if(*s1 > *s2){
			return 1;
		}else{
			return -1;
		}
	}

	if(*s2 == 0) return 0;
	return -1;
}


/*--------------------------------------------------
	 to upper from low case character
--------------------------------------------------*/

int		toupper(int c)
{
	if(c >= 'a' && c <= 'z'){
		return c & ~0x20;
	}
	return c;
}



/*--------------------------------------------------
	 stricmp() compare strings, include unmatch case
--------------------------------------------------*/

int	stricmp(char *s1, char *s2)
{
	while(*s1){
		if(*s2 == 0) return 1;
		if(toupper(*s1) == toupper(*s2)){
			s1++, s2++;
		}else if(toupper(*s1) > toupper(*s2)){
			return 1;
		}else{
			return -1;
		}
	}

	if(*s2 == 0) return 0;
	return -1;
}


/*--------------------------------------------------
	 strncmp() compare strings
--------------------------------------------------*/

int	strncmp(char *s1, char *s2, uint len)
{
	while(*s1 && len){
		if(*s2 == 0) return 1;
		if(*s1 == *s2){
			s1++, s2++;
		}else if(*s1 > *s2){
			return 1;
		}else{
			return -1;
		}
		len--;
	}

	if(len == 0 || *s2 == 0) return 0;
	return -1;
}


/*--------------------------------------------------
	 strnicmp() compare strings, include unmatch case
--------------------------------------------------*/

int	strnicmp(char *s1, char *s2, uint len)
{
	while(*s1 && len){
		if(*s2 == 0) return 1;
		if(toupper(*s1) == toupper(*s2)){
			s1++, s2++;
		}else if(toupper(*s1) > toupper(*s2)){
			return 1;
		}else{
			return -1;
		}
		len--;
	}

	if(len == 0 || *s2 == 0) return 0;
	return -1;
}


/*-----------------------------------------------
	 strcpy() copy string
-----------------------------------------------*/

char *strcpy(char *s1, char *s2)
{
	char	*p;
	p = s1;
	while((*s1++ = *s2++) != 0);
	return p;
}


/*-----------------------------------------------
	 strcat() concatination strings
-----------------------------------------------*/

char *strcat(char *s1, char *s2)
{
	char *p;
	p = s1;
	while(*s1) s1++;
	strcpy(s1, s2);
	return s1;
}


/*-----------------------------------------------
	strchr(char *s, int c)
	c is Zero, find string end.
-----------------------------------------------*/

char	*strchr(char *s, int c)
{
	if(c == 0){
		while(*s) s++;
		return s;
	}

	while(*s){
		if(*s == c) return s;
		s++;
	}
	return NULL;
}


/*---------------------------------------------------
	 strstr() strings match in strings
---------------------------------------------------*/

char	*strstr(char *s1, char *s2)
{
	int		n, len1, len2;
	char	*p1, *p2;

	len1 = strlen(s1);
	len2 = strlen(s2);

	for(n = 0; n < len1 - len2; n++){
		p1 = s1 +n;
		p2 = s2;
		while(*p1 == *p2){
			p1++; p2++;
		}
		if(*p2 == 0) return s1+n;	// found string
	}

	return NULL;
}


/*---------------------------------------------------
	 stristr() 
---------------------------------------------------*/

char	*stristr(char *s1, char *s2)
{
	int		n, len1, len2;
	char	*p1, *p2;

	len1 = strlen(s1);
	len2 = strlen(s2);

	for(n = 0; n < len1 - len2; n++){
		p1 = s1 +n;
		p2 = s2;
		while(toupper(*p1) == toupper(*p2)){
			p1++; p2++;
		}
		if(*p2 == 0) return s1+n;	// found string
	}

	return NULL;
}


/*** end of pic_string.c **************************/
