/***************************************************************************

	File: setup.c

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

	This file is part of IPphone.
	


***************************************************************************/

/** INCLUDES **************************************************************/
#include <p24FJ64GA002.h>
#include "typedef.h"
#include "pic_232.h"
#include "pic_print.h"
#include "pic_delay.h"
#include "pic_string.h"
#include "W5100.h"

#include "sysconf.h"
#include "lcd.h"
#include "setup.h"
#include "key.h"


/** DEFINITION ************************************************************/ 

enum mid_t {
	MID_SIP_IP = 0,	
	MID_NET_MASK,
	MID_GATEWAY,
	MID_PORT_SIP,
	MID_PORT_RTP,
//	MID_MAC_ADDR,
	MID_TELNUM,
	MID_SIP_ID,
	MID_SIP_PSWRD,
	MID_SRV_IP,
	MID_SRV_PORT,
	MID_SRV_DOMAIN,
	MID_EXPIRES,
	MID_SESSION,
	MID_CALLOUT,
};

/** VARIABLES *************************************************************/ 
const struct	{
	char	*name;
	char	*(*get)(void);
	void	(*set)(char *);
	int		len;
} menu[] = {
	{ "IP", 		get_local_sip_ip,   set_local_sip_ip,   IP_LEN+1, },
	{ "NET MASK",	get_net_mask,	    set_net_mask,		IP_LEN+1, },
	{ "GATEWAY IP",	get_gateway_ip,	    set_gateway_ip, 	IP_LEN+1, }, 
	{ "PORT(SIP)",	(char *(*)(void))get_local_sip_port,
					(void (*)(char *))set_local_sip_port, PORT_LEN+1, },
	{ "PORT(RTP)",	(char *(*)(void))get_local_rtp_port,
					(void (*)(char *))set_local_rtp_port, PORT_LEN+1, }, 
//	{ "MAC ADDR",	get_mac_addr,       set_mac_addr,       MAC_LEN+1, },
	{ "TEL. NUMBER",get_tel_num,        set_tel_num,        TEL_LEN+1, },
	{ "SIP ID",		get_sip_id,			set_sip_id,			ID_LEN+1, },
	{ "SIP PSWRD",  get_sip_pswrd,		set_sip_pswrd,		PSWRD_LEN+1, },
	{ "Srvr IP",	get_remote_sip_ip,	set_remote_sip_ip,	IP_LEN+1, },
	{ "Srvr port",	(char *(*)(void))get_remote_sip_port,
					(void (*)(char *))set_remote_sip_port,PORT_LEN+1, },
	{ "Srvr domain",get_remote_domain,  set_remote_domain,  DMN_LEN+1, },
	{ "Reg. expires",(char *(*)(void))get_expires_time,
					(void (*)(char *))set_expires_time, TIME_LEN+1, }, 
	{ "Session time",(char *(*)(void))get_session_time,
					(void (*)(char *))set_session_time, TIME_LEN+1, }, 
	{ "Callout time",(char *(*)(void))get_callout_time,
					(void (*)(char *))set_callout_time, TIME_LEN+1, }, 

};


const char	*key_data[] = {
/*0*/	"0",
/*1*/	"!\"#$%&'()=~|{}+-/*[]?<>:;.,1",
/*2*/	"ABCabc2",
/*3*/	"DEFdef3",
/*4*/	"GHIghi4",
/*5*/	"JKLjkl5",
/*6*/	"MNOmno6",
/*7*/	"PQRSpqrs7",
/*8*/	"TUVtuv8",
/*9*/	"WXYZwxyz9",
/*.*/	".@/",
/*#*/	"#!\"$%&'()=~|{}+-/*[]?<>:;.,",
};


/*==========================================================================
	setup network information
==========================================================================*/

void	setup_network_information(void)
{
	char	*bp;

	read_config_from_pm();
	bp = msg_buf;

	// not yet stored data in Flash Memory ?
	if((((int)*bp) & 0xff)  == 0x00ff){

		// use default value
		//*** set initilal default values ************

		// set local sip/rtp interface default value
		set_local_sip_ip(IP_ADDR);
		set_local_rtp_ip(IP_ADDR);
		set_local_sip_port(SIP_PORT_NUM);
		set_local_rtp_port(RTP_PORT_NUM);

		// set remote(server) sip interface default value
		set_remote_sip_ip(REMOTE_SIP_IP);
		set_remote_sip_port(REMOTE_SIP_PORT);
		set_remote_domain(REMOTE_DOMAIN);

		// set LAN parameter
		set_gateway_ip(GW_ADDR);
		set_net_mask(NET_MASK);
		set_mac_addr(MAC_ADDR);

		// set telphone number/sip id/sip pswrd
		set_tel_num(TEL_NUMBER);
		set_sip_id(SIP_ID);
		set_sip_pswrd(SIP_PSWRD);

		// set timer value
		set_expires_time(EXPIRES_TIME);
		set_session_time(SESSION_TIME);
		set_callout_time(CALLOUT_TIME);

		return;
	}

	// setup data it is read@from Flash
	set_local_sip_ip(bp); 			bp += strlen(bp); bp++;
	set_local_sip_port(atoi(bp));	bp += strlen(bp); bp++;
	set_local_rtp_ip(bp); 			bp += strlen(bp); bp++;
	set_local_rtp_port(atoi(bp));	bp += strlen(bp); bp++;

	set_remote_sip_ip(bp);			bp += strlen(bp); bp++;
	set_remote_sip_port(atoi(bp));	bp += strlen(bp); bp++;
	set_remote_domain(bp);			bp += strlen(bp); bp++;

	set_gateway_ip(bp);		bp += strlen(bp); bp++;
	set_net_mask(bp);		bp += strlen(bp); bp++;
	set_mac_addr(bp);		bp += strlen(bp); bp++;

	set_tel_num(bp);		bp += strlen(bp); bp++;
	set_sip_id(bp);			bp += strlen(bp); bp++;
	set_sip_pswrd(bp);		bp += strlen(bp); bp++;

	set_expires_time(atoi(bp));	bp += strlen(bp); bp++;
	set_session_time(atoi(bp));	bp += strlen(bp); bp++;
	set_callout_time(atoi(bp));	bp += strlen(bp); bp++;

}


/*----------------------------------------------------------------------
	read config data from program memory and stored to msg_buf(work)
----------------------------------------------------------------------*/

void read_config_from_pm(void)
{
	uint low, high;
	char *bp, *endp;
	int	address;

	TBLPAG = 0;
	address = 0xa400;
	bp = msg_buf;
	endp = bp + MSG_BUF_LEN -3;

	while(bp < endp){
		high = __builtin_tblrdh(address);
		low = __builtin_tblrdl(address);
		*bp++ = high & 0xff;
		*bp++ = (low & 0xff00) >> 8;
		*bp++ = low & 0xff;
		address += 2;
	}

}


/*--------------------------------------------------------------------------
	write config data to Program Memory
--------------------------------------------------------------------------*/

void write_config_to_pm(void)
{
	uint low, high;
	char *bp, *endp;
	int	address;

	// print config data to msg_buf
	bp = msg_buf;
	mysprintf(bp, "%s", get_local_sip_ip()); 	bp += strlen(bp); bp++;
	mysprintf(bp, "%d", get_local_sip_port());  bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_local_rtp_ip());  	bp += strlen(bp); bp++;
	mysprintf(bp, "%d", get_local_rtp_port());  bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_remote_sip_ip()); 	bp += strlen(bp); bp++;
	mysprintf(bp, "%d", get_remote_sip_port()); bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_remote_domain()); 	bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_gateway_ip()); 	bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_net_mask()); 	bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_mac_addr()); 	bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_tel_num()); 	bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_sip_id()); 		bp += strlen(bp); bp++;
	mysprintf(bp, "%s", get_sip_pswrd()); 	bp += strlen(bp); bp++;
	mysprintf(bp, "%d", get_expires_time()); bp += strlen(bp); bp++;
	mysprintf(bp, "%d", get_session_time()); bp += strlen(bp); bp++;
	mysprintf(bp, "%d", get_callout_time()); bp += strlen(bp); bp++;

	*bp++ = 0;
	endp = bp;
	bp = msg_buf;

	// Program Memory address is 0xA400
	TBLPAG = 0;
	address = 0xa400;

	// erase memory 
	NVMCON = 0x4042;
	__builtin_tblwtl(address, 0xffff);
	asm("DISI #16");
	__builtin_write_NVM();
    while(NVMCONbits.WR == 1);

	// write to memory by word
	NVMCON = 0x4003; //Perform WORD write
	while(bp < endp){
		high = *bp++;
		low = *bp++;
		low = (low << 8) + *bp++;
		__builtin_tblwth(address, high);
		__builtin_tblwtl(address, low);
		asm("DISI #16");
		__builtin_write_NVM();
        while(NVMCONbits.WR == 1);
		address += 2;
	}

}


/*==========================================================================
	edit configuration
==========================================================================*/

void	edit_config(void)
{
	char edit_buf[DMN_LEN+1];
	int	id;
	
	for(id = 0; id < sizeof(menu)/sizeof(menu[0]); id++){

		//
		// display menu item
		//
		lcd_cmdwrite(cmd_CLEAR);
		lcd_cmdwrite(LCD_SET_LINE1);
		lcd_strwrite(menu[id].name);

		//
		// get parameter now value, and stored to edit buffer
		// 
		switch(id){
		case MID_PORT_SIP:
		case MID_PORT_RTP:
		case MID_SRV_PORT:
		case MID_EXPIRES:
		case MID_SESSION:
		case MID_CALLOUT:
			mysprintf(edit_buf, "%d", (int)menu[id].get() );
			break;
		default:
			strcpy(edit_buf, menu[id].get());
			break;
		}

		// edit 
		if(lcd_line_edit(edit_buf, menu[id].len) == -1){
			return;	// edit cancel
		}

		// check edit result
		if(strlen(edit_buf) == 0){
			id--; continue;	// edit buffer empty, retry edit!
		}

		//
		// check format ip/telnumber
		//
		switch(id){
		case MID_SIP_IP:	
		case MID_NET_MASK:
		case MID_GATEWAY:
		case MID_SRV_IP:
			if(!isip(edit_buf)){
				id--; continue; // error, edit retry!
			}
			break;

		case MID_TELNUM:
			if(!istel(edit_buf)){
				id--; continue;	// error, edit retry!
			}
			break;
		}

		//
		// update parameter
		//   IP,NETMASK,GATEWAY,PORT doesn't effect yet.
		switch(id){
		case MID_PORT_SIP:
		case MID_PORT_RTP:
		case MID_SRV_PORT:
		case MID_EXPIRES:
		case MID_SESSION:
		case MID_CALLOUT:
			menu[id].set((void *)atoi(edit_buf));
			break;
		default:
			menu[id].set(edit_buf);
			break;
		}
	}

	// set W5100 config
	//  IP,NETMASK,GATEWAY,PORT
	w5100_initialize();

	// stored Flash memory
	write_config_to_pm();

}


/*--------------------------------------------------------------------------
	lcd_line_edit()
--------------------------------------------------------------------------*/

int lcd_line_edit(char *buf, int limit)
{
	int		p, edit_pos;
	int		buf_len;
	int		dsp_start;
	int		prev, key, key_mode;
	const char	*kp;


	edit_pos = buf_len = strlen(buf);
	dsp_start = 0;
	key_mode = KMD_NORMAL;
	display_keymode(key_mode);


	for(;;){
		// display control
		//   left over
		while(edit_pos < dsp_start){
			dsp_start--;
		}
		//   right over
		while(edit_pos >= dsp_start+LCD_DSP_LENGTH){
			dsp_start++;
		}

		// display now
		lcd_cmdwrite(LCD_SET_LINE2);
		lcd_strwrite(buf +dsp_start);
		p = (dsp_start + LCD_DSP_LENGTH) - buf_len;
		while(p >= 0){
			p--;
			lcd_datawrite(' ');
		}

		// move cursor to edit position
		lcd_cmdwrite(LCD_SET_LINE2 + (edit_pos - dsp_start));

		// blink on
		lcd_cmdwrite(cmd_DISPLAY | prm_DISPLAY_ON | prm_CURSOR_OFF | prm_BLINK_ON);

		// wait inkey
		while((key = getkey()) == -1){
			delay_ms(10);
		}

		// blink off
		lcd_cmdwrite(cmd_DISPLAY | prm_DISPLAY_ON | prm_CURSOR_OFF | prm_BLINK_OFF);


		switch(key){
		case KEY_ON_HOOK:
			return -1;		// edit cancel

		case KEY_RIGHT:
			// cursor move to right
			if(edit_pos < buf_len){
				edit_pos++;
				prev = KEY_NONE;
			}
			break;

		case KEY_LEFT:
			// cursor move to left
			if(edit_pos > 0){
				edit_pos--;
				prev = KEY_NONE;
			}
			break;

		case KEY_ENTER:
			return 0;	// normal end

		case KEY_CLEAR:
			// back space 1 char at end
			if(buf_len && edit_pos >= buf_len){
				buf[--buf_len] = 0;
				edit_pos = buf_len;
			}else{
			// delete 1 char in buffer
				p = edit_pos;
				do{
					buf[p] = buf[p+1];
					p++;
				}while(p < buf_len); 
				buf_len--;
			}
			prev = KEY_NONE;
			break;

		case KEY_MODE:	// change key input mode
			key_mode ^= 1;	// togle key input mode
			display_keymode(key_mode);
			prev = KEY_NONE;
			break;

		case KEY_OFF_HOOK:	// these keys don't effect
		//case KEY_MENU:
			break;

		default:
			// buffer full ?
			if(buf_len >= limit -1) break;

			//*** input mode is numeric
			if(key_mode == KMD_NORMAL){
				// add 1 char tail
				if(edit_pos >= buf_len){
					buf[buf_len++] = key;
					buf[buf_len] = 0;
					edit_pos = buf_len;
				}else{
				// insert 1 char into buffer
					p = ++buf_len;
					do{
						buf[p] = buf[p-1];
						p--;
					}while(p > edit_pos);
					buf[edit_pos] = key;
					edit_pos++;
				}
				break;
			}

			//*** input mode is alphabetical
			// same prev key
			if(key == prev){
				// change char, on edit position
				buf[edit_pos] = *kp; 
			}else{
				// change key
				if(prev != KEY_NONE) edit_pos++;
				prev = key;
				if(key == '.') kp = key_data[10];
				else if(key == '#') kp = key_data[11];
				else kp = key_data[key - '0'];
				
				// add 1 char tail
				if(edit_pos >= buf_len){
					buf[buf_len++] = *kp;
					buf[buf_len] = 0;
					//edit_pos = buf_len;
				}else{
				// insert 1 char into buffer
					p = ++buf_len;
					do{
						buf[p] = buf[p-1];
						p--;
					}while(p > edit_pos);
					buf[edit_pos] = *kp;
					//edit_pos++;
				}
			}

			kp++;
			if(*kp == 0){
				if(key == '.') kp = key_data[10];
				else if(key == '#') kp = key_data[11];
				else kp = key_data[key - '0'];
			}
			break;

		} // switch

	} // for()

}


/*--------------------------------------------------------------
	display keymode
--------------------------------------------------------------*/

void	display_keymode(int mode)
{
	lcd_cmdwrite(LCD_SET_LINE1 + 13);
	lcd_cmdwrite(cmd_DISPLAY | prm_DISPLAY_ON | prm_CURSOR_OFF | prm_BLINK_ON);
	lcd_strwrite(mode == KMD_NORMAL ? "[N]": "[A]");
	lcd_cmdwrite(cmd_DISPLAY | prm_DISPLAY_ON | prm_CURSOR_OFF | prm_BLINK_OFF);
}


/*--------------------------------------------------------------------------
	check telephone dial number
--------------------------------------------------------------------------*/

int	istel(char *dial)
{
	while(*dial){
		if(*dial < '0' || *dial > '9') return 0;
		dial++;
	}
	return 1;
}


/*--------------------------------------------------------------------------
	check IP address IPv4 only
--------------------------------------------------------------------------*/

int	isip(char *ip)
{
	int	digit_count;
	int	addr;
	int	n;

	addr = atoi(ip);
	if(addr > 255) return 0;
	digit_count = 0;
	while(*ip >= '0' && *ip <= '9'){
		digit_count++;
		ip++;
	}
	if(digit_count == 0 || digit_count > 3) return 0;

	for(n = 0; n < 3; n++){
		if(*ip != '.') return 0;
		ip++;

		addr = atoi(ip);
		if(addr > 255) return 0;
		digit_count = 0;
		while(*ip >= '0' && *ip <= '9'){
			digit_count++;
			ip++;
		}
		if(digit_count == 0 || digit_count > 3) return 0;
	}

	//if(addr == 0) return 0;	don't net mask
	if(*ip) return 0;

	return 1;
}



/*** end of setup.c **********************************************************/
