/*********************************************************************************

	File: sound.c

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

	This file is part of IPphone.

*/

#include <p24FJ64GA002.h>
#include "typedef.h"
#include "pic_print.h"
#include "pic_string.h"
#include "pic_232.h"
#include "pic_timer.h"
#include "w5100.h"

#include "sysconf.h"
#include "message.h"
#include "sound.h"

#define	OFF		0
#define	ON		1
#define	PWM_CENTER		250

#define	RTP_HEAD_SIZE	12
#define	PAYLOAD_SIZE	160
#define BUF_CNT			5


// 400Hz dial tone
const signed char	wave_dial_tone[20] = {
		0,  26,  50,  68,  80,  84,  80,  68,  50,  26,
		0, -26, -50, -68, -80, -84, -80, -68, -50, -26, };

// ring back tone 400Hz 85% AM by 20Hz
const signed char wave_ring_back_tone[400] = {
    0,    26,    49,    68,    80,    84,    79,    68,    49,    26, 
    0,   -25,   -48,   -67,   -78,   -82,   -78,   -66,   -48,   -25, 
    0,    25,    48,    66,    77,    80,    76,    65,    47,    24, 
    0,   -24,   -46,   -63,   -74,   -78,   -74,   -62,   -45,   -24, 
    0,    23,    44,    60,    71,    74,    70,    60,    43,    22, 
    0,   -22,   -42,   -57,   -67,   -70,   -66,   -56,   -40,   -21, 
    0,    20,    39,    54,    62,    65,    62,    52,    37,    20, 
    0,   -19,   -36,   -49,   -57,   -60,   -56,   -48,   -34,   -18, 
    0,    17,    32,    44,    52,    54,    51,    43,    30,    16, 
    0,   -15,   -29,   -40,   -46,   -48,   -45,   -38,   -27,   -14, 
    0,    14,    26,    35,    40,    42,    40,    33,    24,    12, 
    0,   -12,   -22,   -30,   -35,   -36,   -34,   -28,   -20,   -10, 
    0,    10,    18,    26,    29,    30,    28,    24,    17,     8, 
    0,    -8,   -16,   -21,   -24,   -25,   -23,   -19,   -14,    -7, 
    0,     6,    12,    17,    20,    20,    18,    16,    11,     6, 
    0,    -5,   -10,   -13,   -15,   -16,   -14,   -12,    -8,    -4, 
    0,     4,     8,    10,    12,    12,    11,     9,     6,     3, 
    0,    -3,    -6,    -8,    -9,   -10,    -9,    -7,    -5,    -2, 
    0,     2,     4,     6,     7,     8,     7,     6,     4,     2, 
    0,    -2,    -4,    -6,    -6,    -6,    -6,    -5,    -4,    -2, 
    0,     2,     4,     5,     6,     6,     6,     6,     4,     2, 
    0,    -2,    -4,    -6,    -7,    -8,    -7,    -6,    -4,    -2, 
    0,     2,     5,     7,     9,    10,     9,     8,     6,     3, 
    0,    -3,    -6,    -9,   -11,   -12,   -12,   -10,    -8,    -4, 
    0,     4,     8,    12,    14,    16,    15,    13,    10,     5, 
    0,    -6,   -11,   -15,   -18,   -20,   -20,   -17,   -12,    -6, 
    0,     7,    14,    19,    23,    25,    24,    21,    16,     8, 
    0,    -8,   -16,   -24,   -28,   -30,   -29,   -25,   -19,   -10, 
    0,    10,    20,    28,    34,    36,    35,    30,    22,    12, 
    0,   -12,   -24,   -33,   -39,   -42,   -40,   -35,   -26,   -14, 
    0,    14,    27,    38,    45,    48,    46,    40,    29,    16, 
    0,   -16,   -30,   -42,   -50,   -54,   -52,   -44,   -33,   -17, 
    0,    18,    34,    47,    56,    60,    57,    49,    36,    19, 
    0,   -19,   -37,   -52,   -61,   -65,   -62,   -54,   -39,   -21, 
    0,    21,    40,    56,    66,    70,    67,    57,    42,    22, 
    0,   -22,   -42,   -59,   -70,   -74,   -71,   -61,   -44,   -24, 
    0,    24,    45,    62,    74,    78,    74,    64,    46,    24, 
    0,   -24,   -46,   -64,   -76,   -80,   -77,   -66,   -48,   -25, 
    0,    25,    48,    66,    78,    82,    78,    67,    48,    26, 
    0,   -25,   -48,   -67,   -79,   -84,   -80,   -68,   -49,   -26, 
};

// MIC(voice) capture buffer
volatile int mic_raw;	// MIC capture data	(signed 10bits)

// sound on/off control
int 	voice_ctrl;		// voice control
int		ring_back_ctrl;	// ring back control
int		busy_ctrl;		// busy tone control
int		ring_bell_ctrl;	// ring bell control
int		dtmf_ctrl;		// dtmf control

// RTP packet buffer
uchar	rec_voice_buffer[RTP_HEAD_SIZE+PAYLOAD_SIZE];	// for receiving 
uchar	send_voice_buffer[RTP_HEAD_SIZE+PAYLOAD_SIZE];	// for sending

// sound data(voice or tone) playback buffer
uint	play_buffer[BUF_CNT][PAYLOAD_SIZE];
uchar	play_sem[BUF_CNT];			// buffer semaphore
int 	read_buf_idx, read_idx;		// buffer read index
int 	write_buf_idx, write_idx;	// buffer write index

// voice from mic capture buffer 
uint	record_buffer[2][PAYLOAD_SIZE];
uchar	record_sem[2];	// buffer semaphore
int		rcd_buf_idx;	// record buffer index
int 	rcd_write_idx;	// record buffer write index


/*----------------------------------------------------------------------
	Timer T1 interrupt
	Timer T1 8kHz interval timer.
	Pulse wide modulation by voice data or tone signal data.
----------------------------------------------------------------------*/

void __attribute__ ((__interrupt__, no_auto_psv)) _T1Interrupt(void)
{
	IFS0bits.T1IF = 0;		// clear interrupt flag

	//*** Speaker Out ***************************
	// playback buffer not ready ?
	if(!play_sem[read_buf_idx]) goto mic_in;

	// modulate pulse width
	OC1RS = play_buffer[read_buf_idx][read_idx++];

	// buffer index update
	if(read_idx >= PAYLOAD_SIZE){
		read_idx = 0;
		if(voice_ctrl || ring_back_ctrl || ring_bell_ctrl) play_sem[read_buf_idx] = 0;
		if(++read_buf_idx >= BUF_CNT) read_buf_idx = 0;
	}

mic_in:
	//*** MIC in ********************************
	// voice off ?
	if(!voice_ctrl) return;


	// buffer not empty, yet send ?
	if(record_sem[rcd_buf_idx]) return;

	// capture data write to buffer .
	record_buffer[rcd_buf_idx][rcd_write_idx++] = (dtmf_ctrl == OFF) ? mic_raw: dtmf_raw();

	// update buffer index
	if(rcd_write_idx >= PAYLOAD_SIZE){
		rcd_write_idx = 0;
		record_sem[rcd_buf_idx] = 1; // complete write buffer full
		rcd_buf_idx ^= 1;	// change buffer
	}

}


/*==============================================================================
	dial tone
==============================================================================*/

void	dial_tone(int sw)
{
	if(sw == ON){
		for(write_buf_idx = 0; write_buf_idx < BUF_CNT; write_buf_idx++){
			// sema set
			play_sem[write_buf_idx] = 1;
			// tone signal data set
			for(write_idx = 0; write_idx < PAYLOAD_SIZE; write_idx++){
				play_buffer[write_buf_idx][write_idx] = (int)wave_dial_tone[write_idx % 20] + PWM_CENTER;
			}
		}
		read_buf_idx = 0;
		read_idx = 0;
		voice_ctrl = ring_back_ctrl = busy_ctrl = ring_bell_ctrl = OFF;
		speaker(ON);

	}else{
		speaker(OFF);
	}
}


/*==============================================================================
	ring back tone 
==============================================================================*/
static	int	time_count;

void	ring_back_tone(int sw)
{
	if(sw == ON){
		// ring back tone on
		// 	sema clear
		for(write_buf_idx = 0; write_buf_idx < BUF_CNT; write_buf_idx++) play_sem[write_buf_idx] = 0;

		//  clear read pointer
		read_buf_idx = read_idx = 0;

		//  clear control flag
		voice_ctrl = busy_ctrl = ring_bell_ctrl = OFF;

		// set write pointer & control
		ring_back_ctrl = ON;
		write_buf_idx = 0;
		time_count = 0;

		// speaker on
		speaker(ON);
	}else{

		speaker(OFF);

		// ring back tone off
		ring_back_ctrl = OFF;
	}
}

/*------------------------------------------------------------------------------
	generate ring back tone
	400Hz AM85% by 20Hz,  repeat(1sec on, 2sec off)
------------------------------------------------------------------------------*/

void	do_ring_back_tone(void)
{
	int		tone_idx;

	/*
	 *time_count 0-7999	1sec  play
	 *      8000-23999   2sec  stop
	 */

	// when buffer isn't empty, do nothing
	while(play_sem[write_buf_idx] == 0){

		for(write_idx = 0; write_idx < PAYLOAD_SIZE; write_idx++){
			tone_idx = time_count % 400;
			if(time_count < 8000){
				play_buffer[write_buf_idx][write_idx] = (int)wave_ring_back_tone[tone_idx] + PWM_CENTER;
			}else{
				play_buffer[write_buf_idx][write_idx] = PWM_CENTER;
			}
			if(++time_count >= 24000) time_count = 0;
		}

		play_sem[write_buf_idx++] = 1;

		if(write_buf_idx >= BUF_CNT) write_buf_idx = 0;
	}
}


/*==============================================================================
	output busy tone
==============================================================================*/

static	int	busy_tid;
static	int	busy_on;

void	busy_tone(int sw)
{
	if(sw == ON){

		// 400Hz data set
		for(write_buf_idx = 0; write_buf_idx < BUF_CNT; write_buf_idx++){
			// tone signal data set
			for(write_idx = 0; write_idx < PAYLOAD_SIZE; write_idx++){
				play_buffer[write_buf_idx][write_idx] = (int)wave_dial_tone[write_idx % 20] + PWM_CENTER;
			}
			// sema set
			play_sem[write_buf_idx] = 1;
		}
	
		read_buf_idx = 0;
		read_idx = 0;
		ring_back_ctrl = voice_ctrl = ring_bell_ctrl = OFF;
		busy_ctrl = ON;
		// start 0.5sec out
		busy_tid = timer_entry(5, NULL);
		busy_on = 1;
		speaker(ON);
	}else{
		speaker(OFF);
		timer_delete(busy_tid);
		busy_ctrl = OFF;
	}

}

/*------------------------------------------------------------------------------
	generate busy tone
	400Hz repeat(0.5sec out, 0.5sec stop)
------------------------------------------------------------------------------*/

void	do_busy_tone(void)
{
	if(timer_check(busy_tid)) return;

	busy_on ^= 1;
	if(busy_on){
		speaker(ON);
	}else{
		speaker(OFF);
	}
	timer_update(busy_tid, 5);

}


/*==============================================================================
	ring bell sound 
==============================================================================*/

void	ring_bell(int sw)
{
	/*
	 * time_count 0-15999 2sec  play
	 *        16000-23999 1sec  stop
	 */

	if(sw == ON){
		// ring bell on
		// 	sema clear
		for(write_buf_idx = 0; write_buf_idx < BUF_CNT; write_buf_idx++) play_sem[write_buf_idx] = 0;
		//  clear read pointer
		read_buf_idx = read_idx = 0;
		//  clear control flag
		voice_ctrl = busy_ctrl = ring_back_ctrl = OFF;
		// set write pointer & control
		ring_bell_ctrl = ON;
		write_buf_idx = 0;
		time_count = 0;
		// speaker on
		speaker(ON);
	}else{
		speaker(OFF);
		ring_bell_ctrl = OFF;
	}
}

/*------------------------------------------------------------------------------
	generate ring bell
------------------------------------------------------------------------------*/

void	do_ring_bell(void)
{
	int	tone_idx;

	// when buffer isn't empty, do nothing
	while(play_sem[write_buf_idx] == 0){

		for(write_idx = 0; write_idx < PAYLOAD_SIZE; write_idx++){
			tone_idx = time_count % 400;
			if(time_count < 16000){
				play_buffer[write_buf_idx][write_idx] = (int)wave_ring_back_tone[tone_idx] + PWM_CENTER;
			}else{
				play_buffer[write_buf_idx][write_idx] = PWM_CENTER;
			}
			if(++time_count >= 24000) time_count = 0;
		}

		play_sem[write_buf_idx++] = 1;

		if(write_buf_idx >= BUF_CNT) write_buf_idx = 0;
	}
}


/*==============================================================================
	voice 
==============================================================================*/
// for playback voice
static	int	first, rec_flg;

// RTP packet header value
static	uchar	marker, bsel;
static	ushort	seq;
static	ulong	timestamp, ssrc;

void	voice(int sw)
{
	int		n;

	if(sw == ON){

		// sema clear
		for(n = 0; n < BUF_CNT; n++) play_sem[n] = 0; //for play
		record_sem[0] = record_sem[1] = 0;	// for record

		// clear read pointer of playing and  clear write pointer of recording   
		read_buf_idx = 0; read_idx = 0;	// received g77 voice data decode to pwm out
		rcd_buf_idx = 0; rcd_write_idx = 0;	// mic input sampling and data store to buffer
		write_buf_idx = 0;

		// initialize RTP packet header parameter
		marker = 0x80;
		seq = rand();
		timestamp = ((ulong)rand() << 16) | rand();
		ssrc = ((ulong)rand() << 16) | rand();
		bsel = 0;

		// set control
		ring_back_ctrl = busy_ctrl = ring_bell_ctrl = OFF;
		voice_ctrl = ON;
		rec_flg = 0;
		first = 1;

		// start 		
		speaker(ON);

	}else{
		speaker(OFF);
		voice_ctrl = OFF;
	}
}

/*==============================================================================
	voice play and capture
==============================================================================*/

void	do_voice(void)
{
	play_voice();
	send_voice();
}


/*==============================================================================
	decode voice and playback
==============================================================================*/

void	play_voice(void)
{
	int			n;
	static int	size;

	// voice packet already received?
	if(rec_flg) goto buffer_full_check;

	// received voice packet ?
	if((size = RcvRtpPacket((char *)rec_voice_buffer, PAYLOAD_SIZE + RTP_HEAD_SIZE)) <= 0)
		return;


buffer_full_check:
	// next buffer is full?
	if(play_sem[write_buf_idx]){
		rec_flg = 1;
		return;
	}

	rec_flg = 0;

	// decode voice, convert voice data from 8bit(g711) to 9bit(PCM) 	
	for(write_idx = 0, n = RTP_HEAD_SIZE; n < size; write_idx++, n++){
		play_buffer[write_buf_idx][write_idx] = decode_g711(rec_voice_buffer[n]) + PWM_CENTER;
	}

	if(first){
		write_buf_idx++;
		if(write_buf_idx >= (BUF_CNT -1)){
			for(n = 0; n < BUF_CNT -1; n++) play_sem[n] = 1;
			first = 0;
		}
	}else{
		play_sem[write_buf_idx++] = 1;
		if(write_buf_idx >= BUF_CNT) write_buf_idx = 0;
	}

}


/*==============================================================================
	encode and send voice data from record buffer
==============================================================================*/

void	send_voice(void)
{
	uchar	*bp;
	int		n;

	while(record_sem[bsel]){

		// set header
		bp = send_voice_buffer;
		*bp++ = 0x80;
		*bp++ = marker; marker = 0;

		// sequence number
		*bp++ = seq >> 8;
		*bp++ = seq & 0xff;
		seq++;

		// timestamp
		*bp++ = (timestamp >> 24) & 0xff;
		*bp++ = (timestamp >> 16) & 0xff;
		*bp++ = (timestamp >> 8) & 0xff;
		*bp++ = timestamp & 0xff;
		timestamp += PAYLOAD_SIZE;

		// SSRC syncronization source identifire
		*bp++ = (ssrc >> 24) & 0xff;
		*bp++ = (ssrc >> 16) & 0xff;
		*bp++ = (ssrc >> 8) & 0xff;
		*bp++ = ssrc & 0xff;

		// set voice data
		// encode g711, from 10bits to 8bits
		for(n = 0; n < PAYLOAD_SIZE; n++){
			send_voice_buffer[RTP_HEAD_SIZE + n] = encode_g711(record_buffer[bsel][n]);
		}

		// clear sending flag
		record_sem[bsel] = 0;

		// sending to remote
		RtpPacketSend((char *)send_voice_buffer, RTP_HEAD_SIZE+PAYLOAD_SIZE);

		// change buffer
		bsel ^= 1;
	}
}


/*------------------------------------------------------------------------------
	control speaker out 
------------------------------------------------------------------------------*/

void	speaker(int sw)
{
	if(sw == ON){	// speaker on
		// enable PWM output
		OC1R = OC1RS = PWM_CENTER;
		OC1CONbits.OCM = 0b110;
		T3CONbits.TON = 1;	// start pulse genarate
		T1CONbits.TON = 1;	// start pwm

	}else{	// speaker off
		// disable PWM output
		T1CONbits.TON = 0;	// stop pwm
		OC1CONbits.OCM = 0b000;
		T3CONbits.TON = 0;	// stop pulse generate

	}
}


/*------------------------------------------------------------------------------
	g711 u-law decode
------------------------------------------------------------------------------*/

const	uint	g711_seg_offset[8]={0, 33, 99, 231, 495, 1023, 2079, 4191,};

uint	decode_g711(uchar data)
{
	uint	sgn, seg;
	int	val;

	sgn = (data & 0x80);
	data = (~data) & 0x7f;
	seg = (data & 0x70) >> 4;
	data &= 0xf;

	val = ((uint)data << (seg+1)) + g711_seg_offset[seg];
	val >>= 5;	// 14bits to 9bits(+/-256)

	if(!sgn) val = -val;
	return val;
}

/*------------------------------------------------------------------------------
	g711 u-law encode
------------------------------------------------------------------------------*/

uchar   encode_g711(int data)
{
    uchar   code, sgn;
    int		seg;


    // convert 10bits -> 14bits
    data <<= 4;

    if(data < 0){
        sgn = 1;
        data = -data;
    }else{
        sgn = 0;
    }

    // encode g711 (14bits -> 8bits)
    for(seg = 7; seg >= 0; seg--){
        if(data >= g711_seg_offset[seg]){
            code = (seg << 4) + ((data - g711_seg_offset[seg]) >> (seg+1));
            break;
        }
    }

    if(sgn) code |= 0x80;
    return ~code ;
}


/***** end of sound.c *********************************************************/
