/**************************************************************************

	File: sysconf.c

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

	This file is part of IPphone.


setting I/O pin
	( 1)MCLR	ICSP
	( 2)AN0		scan key for keypad key1
	( 3)AN1		scan key for keypad key2
	( 4)AN2		scan key for keypad key3
	( 5)AN3		audio capture from MIC
	( 6)RB2		(NC)
	( 7)RB3/RP3	audio playback from OC1 PWM out
	( 8)Vss		GND
	( 9)RA2		(NC)
	(10)RA3		(NC)
	(11)SOSCI	32.768Khz xtl for clock, (option)
	(12)SOSCO	32.768Khz xtl for clock, (option)
	(13)Vdd		3V3
	(14)PGD3	ICSP
	(15)PGC3	ICSP
	(16)RB7		W5100 LAN module reset
	(17)RB8		LCD module, I2C1 clock,	10K pullup
	(18)RB9		LCD module, I2C1 data,  10k pullup
	(19)DISVREG	    10uF to (20)pin and Vss
	(20)Vcap/Vddcore 10uF to (19)pin
	(21)RB10	U1TX rs232c send
	(22)RB11	U1RX rs232c receive
	(23)RB12	W5100 LAN module, SPI slave select
	(24)RB13	W5100 LAN module, SPI clock out
	(25)RB14	W5100 LAN module, SPI data output (MOSI)
	(26)RB15	W5100 LAN module, SPI data input (MISO)
	(27)Vss		GND
	(28)Vdd		3V3


internal peripheral assign
	SPI1	W5100 LAN module control
	I2C1	LCD module display control
	OC1		PWM out 
	T1		PWM modulate interval timer, interval frequncy is 8Khz.
	T2		100msec interval timer, for application.
	T3		PWM pulse generate clock, frequency is 40Khz.	
	AD1.AN0	keyunit keyscan 1
	AD1.AN1	keyunit keyscan 2
	AD1.AN2	keyunit keyscan 3
	AD1.AN3	audio capture from MIC	

out side peripheral
	LM386	audio amp for Speaker, earphone, 
	NJM2732	MIC amp

***************************************************************************/

/** INCLUDES **************************************************************/
#include <p24FJ64GA002.h>
#include "typedef.h"
#include "pic_232.h"
#include "pic_print.h"
#include "pic_delay.h"
#include "pic_string.h"
#include "pic_timer.h"
#include "W5100.h"
#include "sound.h"
#include "sysconf.h"
#include "lcd.h"
#include "key.h"


/** DEFINITION ************************************************************/ 

/** VARIABLES *************************************************************/ 

// global variables
char	msg_buf[MSG_BUF_LEN];	// send message buffer
char	rx_buf[RX_BUF_LEN];		// receive message buffer
char	saved_msg[MSG_BUF_LEN];	// save buffer

// private variables
static	uchar	local_sip_ip[4];
static	uint	local_sip_port;
static	uchar	local_rtp_ip[4];
static	uint	local_rtp_port;
static	uchar	remote_sip_ip[4];
static	uint	remote_sip_port;
static	uchar	remote_rtp_ip[4];
static	uint	remote_rtp_port;
static	char	remote_domain[DMN_LEN +1];
static	uchar	gateway_ip[4];
static	uchar	net_mask[4];
static	uchar	mac_addr[6];

static	uint	expires_time;
static	uint	session_time;
static	uint	callout_time;

static	char	tel_number[TEL_LEN +1];
static	char	sip_id[ID_LEN +1];
static	char	sip_pswrd[PSWRD_LEN +1];

static	int		sip_sck_fd;
static	int		rtp_sck_fd;

/*======================================================================
	System Initialize ( setup peripheral module ) 
======================================================================*/

void	system_initialize(void)
{
	// Fosc(system clock) 8MHz postscale = 1:1 x4 = 32MHz, Fcy = Fosc/2
	CLKDIV = 0;

	//--- setting interrupt ---------------------------------------
	INTCON1 = 0;			// NSTDIS=0 nesting ok

	//--- setting I/O port ----------------------------------------
	AD1PCFG = 0xffff;	// Analog disable
	LATA = 0x0000; TRISA = 0xffff;	// RA are default input.
	LATB = 0x0000; TRISB = 0xffff;	// RB are default input.

	TRISBbits.TRISB3 = 0;	// RB3/RP3 OC1 PWM out
	TRISBbits.TRISB7 = 0;	// LAN module reset
	LATBbits.LATB7 = 1;		//   reset pin to high level(not reset)
	TRISBbits.TRISB8 = 1;	// I2C1 clock, Vdd pullup
	TRISBbits.TRISB9 = 1;	// I2C1 data, Vdd pullup
	TRISBbits.TRISB10 = 0;	// U1TX
	TRISBbits.TRISB11 = 1;	// U1RX
	TRISBbits.TRISB12 = 0;	// W5100 slave select
	TRISBbits.TRISB13 = 0;	// SPI clock output
	TRISBbits.TRISB14 = 0;	// SPI data output
	TRISBbits.TRISB15 = 1;	// SPI data input

//for TEST
TRISAbits.TRISA2 = 0;
TRISBbits.TRISB2 = 0;


	//--- remapping I/O pin for peripheral ------------------------
	//unlock registers
	__builtin_write_OSCCONL(OSCCON & 0xBF);
	// Assign SPI1 
	RPINR20bits.SDI1R = 15; // RP15(RB15)-> SPI1 Data Input
	RPOR7bits.RP14R = 7; // RP14(RB14)-> SPI1 Data Output(7)
	RPOR6bits.RP13R = 8; // RP13(RB13)-> SPI1 Clock Output(8)
	// Assign UART1	
	RPINR18bits.U1RXR = 11;	// RP11(RB11)->UART1 RX
	RPOR5bits.RP10R = 3;	// RP10(RB10)->UART1 TX(3)
	// Assign OC1 To Pin RP3(RB3)
	RPOR1bits.RP3R = 18;	// OC1(18)
	//lock registers
	__builtin_write_OSCCONL(OSCCON | 0x40);

	//--- setting SPI for W5100 -----------------------------------
	// W5100 Slave Select(chip select) 
	LATBbits.LATB12 = 1;		// RB12 W5100 SS#, to HIGH Level(select)
	// 	SCLOCK normal low or High
	// 	DOUT out fall edg, M25P40 read rising edge
	// 	DIN  rising IN, M25P40 out fall edge
	// 	Normal Low CKP = 0, CKE=1 / Normal High CKP = 1, CKE = 0
	//	SDI data sampling, rising edge, SMP = 0
	SPI1STAT = 0;	// SPI1 is disable
	SPI1CON1bits.DISSCK = 0;
	SPI1CON1bits.DISSDO = 0;
	SPI1CON1bits.MODE16 = 0; // 8bits
	SPI1CON1bits.SMP = 1;
	SPI1CON1bits.CKE = 1;
	SPI1CON1bits.SSEN = 0;	// don't slave
	SPI1CON1bits.CKP = 0;
	SPI1CON1bits.MSTEN = 1;	// master mode
	//SPI1CON1bits.SPRE = 0b111;	// second prescale 1:1
	//SPI1CON1bits.PPRE = 0b01;	// first prescale 16:1
	// 16M/16=1Mhz
	//SPI1CON1bits.SPRE = 0b111;	// second prescale 1:1
	//SPI1CON1bits.PPRE = 0b10;	// first prescale 4:1
	// 16M/4=4Mhz
	SPI1CON1bits.SPRE = 0b110;	// second prescale 2:1
	SPI1CON1bits.PPRE = 0b11;	// first prescale 1:1
	// 16M/2=8Mhz
	SPI1CON2 = 0;
	SPI1STATbits.SPIEN = 1;	// start SPI1

	//--- setting for LCD display module --------------------------
	I2C1CON = 0;	// disable I2C, set parameter default value.
	I2C1BRG = 157;	// Fsc=100KHZ, Fcy=16MHz
					// (Fcy/Fsc - Fcy/10,000,000)-1=16000KHz/100KHz-1.6-1=157

	//--- setting OC1 for audio out to speaker -------------------
	OC1CONbits.OCM = 0;		// disable OC module
	OC1CONbits.OCFLT = 0;	// don't use Fault pin
	OC1CONbits.OCTSEL = 1;	// use TMR3
	OC1R = 250;				// initial duty 50% (250/500) 
	OC1RS = 250;			// initial duty 50% (250/500)
	//OC1CONbits.OCM = 0b110;	// OC1 module enable, mode is PWM.
	OC1CONbits.OCM = 0b000;	// OC1 module disable

	// set Timer3 for PWM Pulse generate 
	T3CON = 0;		// stop timer, prescale 1:1
	TMR3 = 0;
	PR3 = 499;		// PWM pulse frequency is 32KHz, 16000KHz/500=32.0KHz

	//--- setting sound and voice sampling timer -------------------
	T1CON = 0;
	TMR1 = 0;
	PR1 = 1999;		// Sampling Frequency is 8000Hz. 16000Khz/2000=8Khz
	IEC0bits.T1IE = 1;	// enable T1 timer interrupt
	IFS0bits.T1IF = 0;	// clear interrupt flag
	IPC0bits.T1IP = 6;	// priority is level 6.

	//--- setting Analog input ------------------------------------
	AD1CON1bits.ADON = 0;	// turn OFF AD module
	AD1CON1bits.ADSIDL = 0;	// continue module operation in idle mode
	AD1CON1bits.FORM = 0b00;	// unsigned integer(0000 00dd dddd dddd) 
	//AD1CON1bits.FORM = 0b01;	// signed integer(ssss sssd dddd dddd) 
	AD1CON1bits.SSRC = 0b111;	// internal counter end sampling and starts conversion.(auto-convert)
	//AD1CON1bits.ASAM = 0;	// Sampling begins when SAMP bit is set 
	AD1CON1bits.ASAM = 1;	// Sampling begins immediately after last conversion complete. SAMP bit is auto-set.
	AD1CON1bits.SAMP = 0;	// yet sampling
	AD1CON1bits.DONE = 0;	// yet conversion(this bit is status bit)

	AD1CON2bits.VCFG = 0b000;	// Voltage reference select AVdd/AVss
	//AD1CON2bits.CSCNA = 0;	// MUX-A Do not scan inputs
	AD1CON2bits.CSCNA = 1;	// MUX-A Scan inputs
	//AD1CON2bits.BUFS 		// this bit is status bit
	//AD1CON2bits.SMPI = 0;	// interrupt at the completion of conversion for each sample/convert sequence
	AD1CON2bits.SMPI = 3;	// interrupt at the completion of conversion for 4th sample/convert sequence
	AD1CON2bits.BUFM = 0;	// Buffer configured as one 16-word buffer
	AD1CON2bits.ALTS = 0;	// Always uses MUX A input multiplexer settings

	AD1CON3bits.ADRC = 0;	// Clock derived from system clock
	//AD1CON3bits.SAMC = 4;	// Auto sample time is 4 * Tad
	//AD1CON3bits.ADCS = 3;	// AD conversion clock select, Tad = 2Tcy = 125ns

	AD1CON3bits.SAMC = 13;	// Auto sample time is 13 * Tad
	AD1CON3bits.ADCS = 3;	// AD conversion clock select, Tad = 2Tcy
	// Tcy=8Mhz 0.125usec
	// Tad=2Tcy=2*0.125usec=0.25usec
	// (12+13)Tad=25*0.25=6.25usec
	// 6.25*4=25usec

	AD1CHSbits.CH0NB = 0;	// Channel 0 positive input select for MUX B,Vr-
	AD1CHSbits.CH0SB = 0;	// Channel 0 positive input select for MUX B,AN0
	AD1CHSbits.CH0NA = 0;	// Channel 0 positive input select for MUX A,Vr-
	AD1CHSbits.CH0SA = 0;	// Channel 0 positive input select for MUX A,AN0

	//AD1CSSL = 0;	// AD input pin don't scan
	AD1CSSL = 0b0000000000001111;	// AD input pin scan select, AN0-AN3
	AD1PCFG = 0b1111111111110000;	// AN0-AN3 is analog input

	IEC0bits.AD1IE = 1;	// enable AD1 interrupt for conversion done 
	IFS0bits.AD1IF = 0;	// clear interrupt flag
	IPC3bits.AD1IP = 4; // priority is level 4.

	key_int_count = INT_TIMES;
	key_match_count = MATCH_TIMES;
 
	AD1CON1bits.ADON = 1;	// turn ADC on


	//--- setting UART -------------------------------------------
	// see pic_232.c
	rs_initialize();

	//--- setting LCD --------------------------------------------
	// see lcd.c
	lcd_initialize();

	//--- setting application timer ------------------------------
	// see pic_timer.c use T2 timer
 	timer_initialize();

}


/*======================================================================
	network / SIP / information get and set function
======================================================================*/

// get local ip/port
char	*get_local_sip_ip(void)		{ static char buf[16]; return print_ip(buf, local_sip_ip); }
char	*get_local_rtp_ip(void)		{ static char buf[16]; return print_ip(buf, local_rtp_ip); }
uint	get_local_sip_port(void)	{ return local_sip_port; }
uint	get_local_rtp_port(void)	{ return local_rtp_port; }

// get remote ip/port
char	*get_remote_sip_ip(void)	{ static char buf[16]; return print_ip(buf, remote_sip_ip); }
char	*get_remote_rtp_ip(void)	{ static char buf[16]; return print_ip(buf, remote_rtp_ip); }
uint	get_remote_sip_port(void)	{ return remote_sip_port; }
uint	get_remote_rtp_port(void)	{ return remote_rtp_port; }
char	*get_remote_domain(void)	{ return remote_domain; }

// get timer value
uint	get_expires_time(void)		{ return expires_time; }
uint	get_session_time(void)		{ return session_time; }
uint	get_callout_time(void)		{ return callout_time; }

// set local ip/port
void	set_local_sip_ip(char *ip)		{ set_ip(local_sip_ip, ip); }
void	set_local_rtp_ip(char *ip)		{ set_ip(local_rtp_ip, ip); }
void	set_local_sip_port(uint port)	{ local_sip_port = port; }
void	set_local_rtp_port(uint port)	{ local_rtp_port = port; }

// set remote ip/port
void	set_remote_sip_ip(char *ip)		{ set_ip(remote_sip_ip, ip); }
void	set_remote_rtp_ip(char *ip)		{ set_ip(remote_rtp_ip, ip); }
void	set_remote_sip_port(uint port)	{ remote_sip_port = port; }
void	set_remote_rtp_port(uint port)	{ remote_rtp_port = port; }
void	set_remote_domain(char *domain)	{ strcpy(remote_domain, domain); }

// set timer value
void	set_expires_time(uint val)		{ expires_time = val; }
void	set_session_time(uint val)		{ session_time = val; }
void	set_callout_time(uint val)		{ callout_time = val; }

// get telephone number/ sip id, sip pswrd
char	*get_tel_num(void)	{ return tel_number; }
char	*get_sip_id(void)	{ return sip_id; }
char	*get_sip_pswrd(void){ return sip_pswrd; }

// set telephone number/ sip id, sip pswrd
void	set_tel_num(char *num)		{ strcpy(tel_number, num); }
void	set_sip_id(char *id)		{ strcpy(sip_id, id); }
void	set_sip_pswrd(char *pswrd)	{ strcpy(sip_pswrd, pswrd); }

// network information
char	*get_gateway_ip(void)	{ static char buf[16]; return print_ip(buf, gateway_ip); }
char	*get_net_mask(void)		{ static char buf[16]; return print_ip(buf, net_mask); }
char	*get_mac_addr(void)		{ static char buf[18]; return print_mac(buf, mac_addr); }

void	set_gateway_ip(char *ip)	{ set_ip(gateway_ip, ip); }
void	set_net_mask(char *mask)	{ set_ip(net_mask, mask); }
void	set_mac_addr(char *mac)		{ set_mac(mac_addr, mac); }


/*======================================================================
	W5100 LAN interface initialize
======================================================================*/

int		w5100_initialize(void)
{
	// Hardware Reset
	LATBbits.LATB7 = 0;
	delay_us(2);
	LATBbits.LATB7 = 1;
	delay_ms(10);

	// Software Reset
	SetMR(MR_RST);	// Software Reset
	while((GetMR() & MR_RST) == MR_RST);	// reset complete ?

	SetMR(0);	// PING block, PPoE/address auto increment/Indirect bus, all disable.
	SetIMR(0xff);	// all interrupt sets to nonmaskable. set '1' is interrupt enable.
	SetRTR(2000);	// set timeout to 200msec
	SetRCR(3);	// Set Retry Count Register( to 3rd)
	SetRMSR(0x55);	// RX memory allocation is 2k * 4 socket
	SetTMSR(0x55);	// TX memory allocation is 2k * 4 socket

	// set LAN parameter
	SetSIPR(local_sip_ip);
	SetSUBR(net_mask);
	SetGWR(gateway_ip);
	SetSHAR(mac_addr);
	clear_socket_flg();

	// UDP SIP/RTP socket open
	sip_sck_fd = socket(SOCK_MR_UDP, local_sip_port, 0);
	if(sip_sck_fd == -1) return -1;

	rtp_sck_fd = socket(SOCK_MR_UDP, get_local_rtp_port(), 0);
	if(rtp_sck_fd == -1){
		closesocket(sip_sck_fd);
		return -1;
	}

	return 0;

}


/*======================================================================
	send_message_with_retry()
	0 -> 0.5 -> 1 -> 2 -> 4,  4times send, wait 7.5sec
======================================================================*/

int		send_message_with_retry(char *msg)
{
	int	size;
	int	tid, tval;
	int	times;

	tval = 5;	// 500msec
	for(times = 0; times < 4; times++){

		// sending message 
		SipMsgSend(msg);

		// wait for server response
		//    set wait timer
		tid = timer_entry(tval, NULL);
		tval *= 2;

		// wait receive
		while( timer_check(tid) ){
			size = RcvSipMsg(rx_buf, RX_BUF_LEN);
			if(size != 0){
				timer_delete(tid);
				return size;	// error = -1, received > 0
			}
		}
		timer_delete(tid);
	}
	return -1;	// time out!!
}


/*======================================================================
	receive_message_with_timeout()
======================================================================*/

int		receive_message_with_timeout(void)
{
	int	size;
	int	tid;

	// wait for server response
	//    set wait timer
	tid = timer_entry(1200, NULL);	// 120sec

	// wait receive
	while(timer_check(tid)){
		size = RcvSipMsg(rx_buf, RX_BUF_LEN);
		if(size != 0){
			timer_delete(tid);
			return size;	// error = -1, received > 0
		}
		//tsk_chg();
	}
	timer_delete(tid);
	return -1;	// time out!!
}


/*======================================================================
	sending sip message
======================================================================*/

int		SipMsgSend(char *msg)
{
	int		size;

	size = sendto(sip_sck_fd, (uchar *)msg, strlen(msg), BLOCK, remote_sip_ip, remote_sip_port);
	if(size == -1) return -1;
	return 0;
}


/*======================================================================
	receiving sip message
	this function called by polling method.
	return code
		-1 error, 0 not exists, >0 received data length
======================================================================*/

int		RcvSipMsg(char *buf, uint buf_len)
{
	int		size;
	uchar	from_ip[4];
	uint	from_port;

	// exists received message ?
	if(GetSnRXRSR(sip_sck_fd) == 0) return 0;

	// receiving message
	size = recvfrom(sip_sck_fd, (uchar *)buf, buf_len-1, NONE_BLOCK, from_ip, &from_port);
	if(size == -1) return -1;
	return size;
}

void	flush_sip_buffer(void)
{
	while(RcvSipMsg(rx_buf, RX_BUF_LEN) > 0){
		delay_ms(20);
	}	
}


/*======================================================================
	sending rtp packet
======================================================================*/

int		RtpPacketSend(char *buf, uint buf_len)
{
	int		size;

	size = sendto(rtp_sck_fd, (uchar *)buf, buf_len, BLOCK, remote_rtp_ip, remote_rtp_port);
	if(size == -1) return -1;
	return 0;
}


/*======================================================================
	receiving rtp packet
	this function called by polling method.
	return code
		-1 error, 0 not yet received packet, >0 received packet length
======================================================================*/

int		RcvRtpPacket(char *buf, uint buf_len)
{
	int		size;
	uchar	from_ip[4];
	uint	from_port;

	// exists received packet ?
	if(GetSnRXRSR(rtp_sck_fd) == 0) return 0;

	// receiving message
	size = recvfrom(rtp_sck_fd, (uchar *)buf, buf_len, NONE_BLOCK, from_ip, &from_port);
	if(size == -1) return -1;

	return size;
}

void	flush_rtp_buffer(void)
{
	while(RcvRtpPacket(rx_buf, RX_BUF_LEN) > 0){
		delay_ms(20); // wait 20msec
	}
}


/*----------------------------------------------------------------------
	TRAP ROUTINE for debug
----------------------------------------------------------------------*/

void __attribute__ ((__interrupt__, no_auto_psv)) _AddressError(void)
{
	INTCON1bits.ADDRERR = 0;
	//myprintf("*** Trap Address error\r\n");
	lcd_cmdwrite(LCD_SET_LINE1);
	lcd_strwrite("Trap Addr. error");

	for(;;);
}

void __attribute__ ((__interrupt__, no_auto_psv)) _MathError(void)
{
	INTCON1bits.MATHERR = 0;
	//myprintf("*** Trap Matherror\r\n");
	lcd_cmdwrite(LCD_SET_LINE1);
	lcd_strwrite("Trap Math error");
	for(;;);
}


/*** end of sysconf.c ********************************************************/
