/**********************************************************************

	sysconf.h

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

	This file is part of IPphone.

**/

#include "typedef.h"

#ifndef	__SYSCONF_H
#define	__SYSCONF_H

// *** IP phone configuration default value ***********
// network information
#define	IP_ADDR				"192.168.1.20"
#define	NET_MASK			"255.255.255.0"
#define	GW_ADDR				"192.168.1.10"
#define	MAC_ADDR			"02.00.00.00.00.01" // local MAC for test.

// SIP/RTP port number 
#define	SIP_PORT_NUM		5060
#define	RTP_PORT_NUM		5800

// SIP proxy server information
#define	REMOTE_SIP_IP		"192.168.1.10"		
#define	REMOTE_SIP_PORT		5060
#define REMOTE_DOMAIN		"192.168.1.10"

// IP phone tel number/SIP id/SIP pswrd
#define	TEL_NUMBER			"5963"
#define	SIP_ID				"id5963"
#define	SIP_PSWRD			"pswd5963"

//  Timer value (sec)
#define	EXPIRES_TIME		3600	// max 10000sec
#define	SESSION_TIME		180		// max  5000sec
#define	CALLOUT_TIME		120		// max  5000sec

// *** CONSTANT definition *******************************
// length of work buffer 
#define	IP_LEN          15
#define	PORT_LEN        5
#define	TEL_LEN         11
#define	ID_LEN          11
#define	PSWRD_LEN       11
#define	DMN_LEN         19
#define	MAC_LEN         17
#define	TIME_LEN        5
#define	MSG_BUF_LEN     1000
#define	RX_BUF_LEN      MSG_BUF_LEN
#define	HEADER_BUF_LEN  128

// *** global variable ***********************************
extern	char            msg_buf[MSG_BUF_LEN];
extern	char            rx_buf[RX_BUF_LEN];
extern	char            saved_msg[MSG_BUF_LEN];
extern	volatile int    key_raw[3];
extern	volatile int    mic_raw;	

// *** function prototype ********************************
// get network parameter
extern	char	*get_gateway_ip(void);
extern	char	*get_net_mask(void);
extern	char	*get_mac_addr(void);

// set network parameter
extern	void	set_gateway_ip(char *ip);
extern	void	set_net_mask(char *mask);	// set_net_mask("255.255.255.0");
extern	void	set_mac_addr(char *mac);	// set_mac_addr("12.34.56.78.9a.bc");

// get local information
extern	char	*get_local_sip_ip(void);
extern	char	*get_local_rtp_ip(void);
extern	uint	get_local_sip_port(void);
extern	uint	get_local_rtp_port(void);

// get remote information
extern	char	*get_remote_sip_ip(void);
extern	char	*get_remote_rtp_ip(void);
extern	uint	get_remote_sip_port(void);
extern	uint	get_remote_rtp_port(void);
extern	char	*get_remote_domain(void);

// set local information
extern	void	set_local_sip_ip(char *ip);	// set_local_sip_ip("192.168.1.20");
extern	void	set_local_rtp_ip(char *ip);
extern	void	set_local_sip_port(uint port);	// set_local_sip_port(5060);
extern	void	set_local_rtp_port(uint port);

// set remote information
extern	void	set_remote_sip_ip(char *ip);
extern	void	set_remote_rtp_ip(char *ip);
extern	void	set_remote_sip_port(uint port);
extern	void	set_remote_rtp_port(uint port);
extern	void	set_remote_domain(char *domain);

// get SIP parameter
extern	char	*get_tel_num(void);
extern	char	*get_sip_id(void);
extern	char	*get_sip_pswrd(void);
extern	uint	get_expires_time(void);
extern	uint	get_session_time(void);
extern	uint	get_callout_time(void);

// set SIP parameter
extern	void	set_tel_num(char *num);
extern	void	set_sip_id(char *id);
extern	void	set_sip_pswrd(char *pswrd);
extern	void	set_expires_time(uint val);
extern	void	set_session_time(uint val);
extern	void	set_callout_time(uint val);

// initializer 
extern	void	system_initialize(void);
extern	int		w5100_initialize(void);

// transfer SIP message 
extern	int		send_message_with_retry(char *msg);
extern	int		receive_message_with_timeout(void);
extern	int		SipMsgSend(char *msg);
extern	int		RcvSipMsg(char *buf, uint buf_len);
extern	int		RcvRtpPacket(char *buf, uint buf_len);
extern	int		RtpPacketSend(char *buf, uint buf_len);

// buffer flush
extern	void	flush_sip_buffer(void);
extern	void	flush_rtp_buffer(void);


#endif	// __SYSCONF_H

/*** end of sysconf.h **************************************/
