/***************************************************************************

	File: ua.c

	Copyright (C) 2012 S.Suwa <www.suwa-koubou.jp>

	This file is part of IPphone.

	Ua is SIP user agent.

***************************************************************************/

/** INCLUDES **************************************************************/
#include <p24FJ64GA002.h>
#include "typedef.h"
#include "pic_232.h"
#include "pic_print.h"
#include "pic_delay.h"
#include "pic_string.h"
#include "pic_timer.h"
#include "W5100.h"

#include "sysconf.h"
#include "sound.h"
#include "message.h"
#include "ua.h"
#include "setup.h"
#include "lcd.h"
#include "key.h"

#define	display_clear()	lcd_cmdwrite(cmd_CLEAR)
extern	void	display_title(void);
extern	void	display_status(void);
extern	void	dispatch(void);
extern	void	write_redial(char *dial);
extern	char	*read_redial(void);
extern	void	st_disconnect_on_hook(void);
extern	void	st_disconnect_off_hook(void);
extern	void	st_calling(void);
extern	void	st_canceling(void);
extern	void	st_progress(void);
extern	void	st_ringing(void);
extern	void	st_connect(void);
extern	void	st_incomming(void);
extern	void	getdspname(void);
extern	int		SndPrack(void);
extern	int		SndBye(void);
extern	int		SndUpdate(void);
extern	int		check_200_status(char *buf, char *method);
extern	void	anlys_200OK(void);
extern	void	send_REGISTER(void);
extern	int		SndRegister(void);

enum state_t {
	STATE_DISCONNECT_ON_HOOK = 0,
	STATE_DISCONNECT_OFF_HOOK,
	STATE_CALLING,
	STATE_CANCELING,
	STATE_PROGRESS,
	STATE_RINGING,
	STATE_CONNECT,
	STATE_INCOMMING,
};

void	(*stfnc[])(void) = {
	st_disconnect_on_hook,
	st_disconnect_off_hook,
	st_calling,
	st_canceling,
	st_progress,
	st_ringing,
	st_connect,
	st_incomming,
};

const char	*status_name[] = {
	"disconnect!!    ",
	"disconnect!!    ",
	"calling ...     ",
	"canceling ...   ",	
	"progress ...    ",
	"ringing ...     ",
	"connect!!       ",
	"ringing ...     ",
};

#define	MAX_DIAL	5

static int	tel_state;	// IPphone state

static char	dial[TEL_SIZE];
static char	tel[TEL_SIZE];
static char	invite_branch[BRANCH_SIZE];
static int	invite_cmdseq;
static char	auth[AUTH_SIZE];

static int	call_type;	// CALLER or CALLEE
static int	reg_tmr;	// REGISTER send timer
static int	update_sender;	// =1 sender, =0 not sender
static int	timer_opt;	// session timer =1 avail, =0 none 

static char	redial[MAX_DIAL][TEL_SIZE];	// redial buffer
static int	widx, nums, ridx;		

/*=========================================================================
	dispatcher
=========================================================================*/

void	dispatch(void)
{
	reg_tmr = -1;
	tel_state = STATE_DISCONNECT_ON_HOOK;

	display_title();
	for(;;){
		display_status();
		(*stfnc[tel_state])();
	}

}


/*-------------------------------------------------------------------------
	display title
-------------------------------------------------------------------------*/

void	display_title(void)
{
	lcd_cmdwrite(LCD_SET_LINE1);
	lcd_strwrite(" << IP Phone >> ");
}


/*-------------------------------------------------------------------------
	display status
-------------------------------------------------------------------------*/

void	display_status(void)
{
	lcd_cmdwrite(LCD_SET_LINE2);
	lcd_strwrite((char *)status_name[tel_state]);
}


/*-------------------------------------------------------------------------
	write dial number to redial buffer 
-------------------------------------------------------------------------*/

void	write_redial(char *dial)
{
	int		idx, n;

	for(idx = widx -1, n = 0; n < nums; idx--, n++){
		if(idx < 0) idx = MAX_DIAL -1;
		if(strcmp(redial[idx], dial) == 0){
			ridx = widx;
			return;	// dial number exists!
		}
	}

	strcpy(redial[widx++], dial);
	widx %= MAX_DIAL; 
	if(nums < MAX_DIAL) nums++;	
	ridx = widx;
}


/*-------------------------------------------------------------------------
	read dial number from redial buffer 
-------------------------------------------------------------------------*/

char	*read_redial(void)
{
	// redial buffer is empty ?
	if(nums == 0) return NULL;

	do{
		ridx--;
		if(ridx < 0) ridx = MAX_DIAL -1;
	}while(redial[ridx][0] == 0);
	return redial[ridx];

}


/*=========================================================================
	state DISCONNECT  On-Hook
	initial state
=========================================================================*/

void	st_disconnect_on_hook(void)
{
	char	*tp, *msg, *rdl, *p;
	int		key, tlen, size, off_hook;

	icon_telephone(OFF);

	flush_sip_buffer();
	flush_rtp_buffer();

	tp = dial; *tp = 0; tlen = 0;
	off_hook = 0;

	for(;;){

		// now is REGISTER send timing ?
		if(reg_tmr != -1 && timer_check(reg_tmr) == 0){
			SndRegister();
			timer_update(reg_tmr, get_expires_time()/2 * 10); // sec -> 0.1sec
			//timer_update(reg_tmr, EXPIRES_TIME /2 * 10); // sec -> 0.1sec
		}

		// check receive initINVITE
		if((size = RcvSipMsg(rx_buf, RX_BUF_LEN)) > 0){
			rx_buf[size] = 0;
			if(strncmp(rx_buf, "INVITE ", 7) == 0 && strstr(rx_buf, get_tel_num()) != NULL){
				// has Require header ?
				if(stristr(rx_buf, "Require:") != NULL){
					msg = make_420_msg(); 
					SipMsgSend(msg);
					continue;
				// has Supported timer ?
				}else if((p = stristr(rx_buf, "Supported:")) != NULL && stristr(p, "timer") != NULL){
					if(((p = stristr(rx_buf, "Session-Expires:")) != NULL) && 
						(atoi(p+strlen("Session-Expires:")) < get_session_time())){
						msg = make_422_msg();
						SipMsgSend(msg);
						continue;
					}
				}
				tel_state = STATE_INCOMMING;
				return;
			}
		}

		// check key press
		key = getkey();

		switch(key){
		case KEY_ENTER:	// register send
			if(!off_hook){
				send_REGISTER();
				tp = dial; *tp = 0; tlen = 0;
				display_title();		
			}
			break;

		case KEY_RIGHT:	// select redial number
			rdl = read_redial();
			if(rdl){
				strcpy(dial, rdl);
				tlen = strlen(dial);
				tp = dial + tlen;
				lcd_cmdwrite(LCD_SET_LINE2);
				lcd_strwrite("                ");
				lcd_cmdwrite(LCD_SET_LINE2);
				lcd_strwrite(dial);
			}
			break;

		case KEY_MODE:	// edit config
			if(off_hook) break;
			edit_config();
			display_clear();
			display_title();
			display_status();
			tp = dial; *tp = 0; tlen = 0;
			break;

		case KEY_ON_HOOK:
			icon_telephone(OFF);
			if(off_hook){
				dial_tone(OFF);
				off_hook = 0;
			}
			if(tlen){
				tp = dial; *tp = 0; tlen = 0;
				display_clear();
				display_title();
			}
			break;

		case KEY_CLEAR:
			if(tlen){
				tlen--;
				tp--;
				*tp = 0;
				lcd_cmdwrite(LCD_SET_LINE2);
				lcd_strwrite(dial);
				lcd_datawrite(' ');
			}
			break;

		case KEY_OFF_HOOK:

			icon_telephone(ON);

			// ready dial number ?
			if(tlen){
				dial_tone(OFF);
				write_redial(dial);
				tel_state = STATE_CALLING;
				return;
			}

			// now dialing
			if(!off_hook){
				dial_tone(ON);
				off_hook = 1;
			}
			break;

		default:
			if(key >= '0' && key <= '9' && tlen < (TEL_SIZE-1)){
				if(tlen == 0){
					lcd_cmdwrite(LCD_SET_LINE2);
					lcd_strwrite("                ");
				}
				*tp++ = key; *tp = 0;
				tlen++;
				lcd_cmdwrite(LCD_SET_LINE2);
				lcd_strwrite(dial);
			}
			break;

		}// switch(key)	

	}// for(;;)
}


/*=========================================================================
	state DISCONNECT  Off-Hook
=========================================================================*/

void	st_disconnect_off_hook(void)
{
	int	tmr;


	tmr = timer_entry(60 * 10, NULL);

	busy_tone(ON);

	for(;;){

		do_busy_tone();

		flush_sip_buffer();
		flush_rtp_buffer();

		if(getkey() == KEY_ON_HOOK || timer_Up(tmr)){
			busy_tone(OFF);
			timer_delete(tmr);
			tel_state = STATE_DISCONNECT_ON_HOOK;
			return;
		}

	}

}


/*=========================================================================
	state CALLING (outgoing call)
=========================================================================*/

void	st_calling(void)
{
	char	*msg, *p;
	char	from_tag[TAG_SIZE];
	char	call_id[CALL_ID_SIZE];
	char	*to_tag;
	int		retry_407;
	int		size, tid, tval, times;

call_retry:

	// copy dial number to work
	strcpy(tel, dial);

	// clear message buffer
	saved_msg[0] = 0;

	// set parameter
	make_branch(invite_branch);
	make_tag(from_tag);
	make_call_id(call_id);
	invite_cmdseq = make_seq();
	to_tag = NULL;	
	retry_407 = 0;
	call_type = CALLER;
	update_sender = 0;

	// make initial INVITE message
	msg = make_INVITE_msg(tel, invite_branch, from_tag, to_tag, call_id, invite_cmdseq, NULL);

msg_send:	// 407 retry 

	// wait for 1xx, with retry
	for(tval = 5, times = 0; times < 4; times++){

		// send INVITE message
		SipMsgSend(msg);

		// wait for server response
		//    set wait timer
		tid = timer_entry(tval, NULL); tval *= 2;
		while(timer_check(tid)){
			if(getkey() == KEY_ON_HOOK){
				timer_delete(tid);
				tel_state = STATE_DISCONNECT_ON_HOOK;
				return;
			}
			if((size = RcvSipMsg(rx_buf, RX_BUF_LEN)) > 0) break;
		}
		timer_delete(tid);

		// check received SIP message
		if(size > 0){
			rx_buf[size] = 0;

			// incomming ?
			if(strncmp(rx_buf, "INVITE ", 7) == 0){
				// send 486 Busy Here
				msg = make_486_msg();
				SipMsgSend(msg);
			}

			if(strncmp(rx_buf, "SIP/2.0 ", 8) == 0){
				break; // break for()
			}

		}// if()

	}// for()

	// invite times out!!
	if(times >= 4) goto do_disconnect;

	// find 407 Proxy Authentication Required
	if(strncmp(rx_buf, "SIP/2.0 407", 11) == 0){

		// make error_ACK message
		msg = make_ACK_msg(tel, NULL, invite_cmdseq);	// same branch

		make_auth(auth, "INVITE");

		// send 407 Ack
		SipMsgSend(msg);

		if(retry_407){
			goto do_disconnect;
		}
		retry_407++;
 
		make_branch(invite_branch);	// update transaction
		invite_cmdseq = make_seq();	// update sequence number
		msg = make_INVITE_msg(tel, invite_branch, from_tag, to_tag, call_id, invite_cmdseq, auth);
		goto msg_send;
	}


	// find 100 Trying
	if(strncmp(rx_buf, "SIP/2.0 100", 11) == 0){

		// saved 100 message for make CANCEL message 
		memory_copy((uchar *)saved_msg, (uchar *)rx_buf, MSG_BUF_LEN);

next_response:
		// wait for next response
		tid = timer_entry(get_callout_time() * 10, NULL);
		//tid = timer_entry(CALLOUT_TIME * 10, NULL);
		while(timer_check(tid) && (size = RcvSipMsg(rx_buf, RX_BUF_LEN)) <= 0){
			if(getkey() == KEY_ON_HOOK){
				timer_delete(tid);
				tel_state = STATE_CANCELING;
				return;
			}
		}
		timer_delete(tid);
		if(size <= 0) goto do_disconnect; // timeout!!
		rx_buf[size] = 0;

		if(strncmp(rx_buf, "INVITE ", 7) == 0){
			// send 486 Busy Here
			msg = make_486_msg();
			SipMsgSend(msg);
			goto next_response;

		// ignore received other Request(not response).
		}else if(strncmp(rx_buf, "SIP/2.0 ", 8) != 0){
			goto next_response;
		}
	}

	// find 183 Session Progress
	if(strncmp(rx_buf, "SIP/2.0 183", 11) == 0){
		// change status
		tel_state = STATE_PROGRESS;
		return;
	}

	// find 180 Ringing
	if(strncmp(rx_buf, "SIP/2.0 180", 11) == 0){
		// change status
		tel_state = STATE_RINGING;
		return;
	}

	// find 200OK, 
	if(check_200_status(rx_buf, "INVITE")){
		// analyse 200OK
		anlys_200OK();
		// change status
		tel_state = STATE_CONNECT;
		return;
	}

	// received error response!!
	// make error_ACK message
	msg = make_ACK_msg(tel, NULL, invite_cmdseq);	// same branch
	SipMsgSend(msg);

	// find 422 Session Interval Too Small
	if(strncmp(rx_buf, "SIP/2.0 422", 11) == 0){
		// get Min-Se and change session timer value
		p = stristr(rx_buf, "Min-SE:");
		if(p != NULL){
			p += 7;	// strlen("Min-SE:") is 7.
			set_session_time(atoi(p));
			goto call_retry;
		}
	}

do_disconnect:
	// change status
	tel_state = STATE_DISCONNECT_OFF_HOOK;
	return;

}


/*=========================================================================
	state PROGRESS
=========================================================================*/

void	st_progress(void)
{
	int		size, tid;
	char	*msg, *hdp;

next_183:

	// saved 183 sip message for make CANCEL message 
	memory_copy((uchar *)saved_msg, (uchar *)rx_buf, MSG_BUF_LEN);

	// set voice 
	anlys_sdp();


	// require 100 rel ?
	if((hdp = stristr(rx_buf, "Require")) != NULL  &&  stristr(hdp, "100rel") != NULL ){
		// send PRACK and receive 200OK(PRACK)
		if(SndPrack() == -1) goto do_disconnect;
	}

	// voice start
	voice(ON);


	// wait for next response during 120sec(default value)
	tid = timer_entry(get_callout_time() * 10, NULL);
	//tid = timer_entry(CALLOUT_TIME *10, NULL);


	for(;timer_check(tid);){

		// doing voice 
		do_voice();


		if(getkey() == KEY_ON_HOOK){
			tel_state = STATE_CANCELING;
			break;
		}

		// doing voice 
		do_voice();

		if((size = RcvSipMsg(rx_buf, RX_BUF_LEN)) > 0){
			rx_buf[size] = 0;

			// find 180 Ringing
			if(strncmp(rx_buf, "SIP/2.0 180", 11) == 0){
				// change status
				tel_state = STATE_RINGING;
				break;

			// next new 183 progress
			}else if(strncmp(rx_buf, "SIP/2.0 183", 11) == 0){
				timer_delete(tid);
				voice(OFF);
				goto next_183;			

			// find 200OK, 
			}else if(check_200_status(rx_buf, "INVITE")){
				anlys_200OK();
				tel_state = STATE_CONNECT;
				break;

			}else if(strncmp(rx_buf, "SIP/2.0 ", 8) == 0){
				// other response 4xx, 6xx etc
				// make error_ACK message
				msg = make_ACK_msg(tel, NULL, invite_cmdseq);	// same branch
				SipMsgSend(msg);
				break;

			}else if(strncmp(rx_buf, "INVITE ", 7) == 0){
				// send 486 Busy Here
				msg = make_486_msg();
				SipMsgSend(msg);
			}else{
				// ignore other REQUEST
				;
			}
		}

	}
	if(!timer_check(tid)) tel_state = STATE_CANCELING;
	timer_delete(tid);
	voice(OFF);
	if(tel_state != STATE_PROGRESS) return;

do_disconnect:
	tel_state = STATE_DISCONNECT_OFF_HOOK;
	return;
}


/*=========================================================================
	state RINGING
=========================================================================*/

void	st_ringing(void)
{
	int		size, tid;
	char	*msg, *hdp;

	// saved 180 message for make CANCEL message 
	memory_copy((uchar *)saved_msg, (uchar *)rx_buf, MSG_BUF_LEN);

	// require 100 rel ?
	if((hdp = stristr(rx_buf, "Require:")) != NULL  &&  stristr(hdp, "100rel") != NULL ){
		// send PRACK and receive 200OK(PRACK)
		if(SndPrack() == -1) goto do_disconnect;
	}

	ring_back_tone(ON);

	// wait for next response during 120sec(default value)
	tid = timer_entry(get_callout_time() * 10, NULL);
	//tid = timer_entry(CALLOUT_TIME, NULL);
	for(;timer_check(tid);){

		// doing ring_back_tone 
		do_ring_back_tone();

		if(getkey() == KEY_ON_HOOK){
			tel_state = STATE_CANCELING;
			break;
		}

		if((size = RcvSipMsg(rx_buf, RX_BUF_LEN)) > 0){
			rx_buf[size] = 0;

			// find 200OK, 
			if(check_200_status(rx_buf, "INVITE")){
				anlys_200OK();
				tel_state = STATE_CONNECT;
				break;

			}else if(strncmp(rx_buf, "SIP/2.0 ", 8) == 0){
				// other response 4xx, 6xx etc
				// make error_ACK message
				msg = make_ACK_msg(tel, NULL, invite_cmdseq);	// same branch
				SipMsgSend(msg);
				break;

			}else if(strncmp(rx_buf, "INVITE ", 7) == 0){
				// send 486 Busy Here
				msg = make_486_msg();
				SipMsgSend(msg);
			}else{
				// ignore other REQUEST
				;
			}
		}

	}
	if(!timer_check(tid)) tel_state = STATE_CANCELING;
	timer_delete(tid);
	ring_back_tone(OFF);
	if(tel_state != STATE_RINGING) return;

do_disconnect:
	tel_state = STATE_DISCONNECT_OFF_HOOK;
	return;
}


/*------------------------------------------------------------------------------
	SndPrack()
------------------------------------------------------------------------------*/

int		SndPrack(void)
{
	char	branch[BRANCH_SIZE], *msg;
	int		cmdseq, size;

	make_branch(branch);
	cmdseq = make_seq();

	msg = make_PRACK_msg(tel, branch, cmdseq, invite_cmdseq);
	size = send_message_with_retry(msg);
	if(size == -1) return -1;	// error!! or time out!!
	rx_buf[size] = 0;
	if(check_200_status(rx_buf, "PRACK")) return 0;
	return -1;	// received other message.
}


/*=========================================================================
	state CANCELING
=========================================================================*/

void	st_canceling(void)
{
	char	*msg;
	int		size, tid;

	memory_copy((uchar *)rx_buf, (uchar *)saved_msg, MSG_BUF_LEN);
	msg = make_CANCEL_msg(tel, invite_branch, invite_cmdseq);

	if((size = send_message_with_retry(msg)) == -1){
		tel_state = STATE_DISCONNECT_ON_HOOK;
		return;
	}

	// check 200OK!!
	rx_buf[size] = 0;
	if(! check_200_status(rx_buf, "CANCEL")){
		tel_state = STATE_DISCONNECT_ON_HOOK; // receive error!!
		return;
	}

	// wait 487 during 2sec
	tid = timer_entry(20, NULL);
	while(timer_check(tid) && (size = RcvSipMsg(rx_buf, RX_BUF_LEN)) <= 0){
			;
	}
	timer_delete(tid);

	if(size > 0){
		rx_buf[size] = 0;
		// make error_ACK message
		msg = make_ACK_msg(tel, NULL, invite_cmdseq);	// same branch
		SipMsgSend(msg);
	}

	tel_state = STATE_DISCONNECT_ON_HOOK;
	return;
}


/*-------------------------------------------------------------------------
	analyse 200 OK response and ACK send
-------------------------------------------------------------------------*/

void	anlys_200OK(void)
{
	char	*msg, *p, branch[BRANCH_SIZE];


	// saved 200OK message for make BYE message 
	memory_copy((uchar *)saved_msg, (uchar *)rx_buf, MSG_BUF_LEN);

	// make ACK(200) message
	make_branch(branch);
	msg = make_ACK_msg(tel, branch, invite_cmdseq);	// new branch
	anlys_sdp();

	// send Ack
	SipMsgSend(msg);


	// check timer option.
	//  required timer ?
	if((p = stristr(rx_buf, "Require:")) != NULL && stristr(p, "timer") != NULL){
		timer_opt = 1;
		//  exists Session-Expires and refresher parameter ?
		if((p = stristr(rx_buf, "Session-Expires:")) != NULL && stristr(p, "refresher=uac") != NULL){
			update_sender = 1;
			// when remote unsupport UPDATE, can't use timer option.
			//  IPphone can't use reINVITE refresh message, it's UPDATE only.
			if((p = stristr(rx_buf, "Allow:")) == NULL || stristr(p, "UPDATE") == NULL){
				timer_opt = 0;
				update_sender = 0;
			}
		}else{
			update_sender = 0;
		}
	}else{
		timer_opt = 0;
		// don't support timer
		update_sender = 0;	// not send UPDATE
	}

}


/*=========================================================================
	state INCOMMING
=========================================================================*/

void	st_incomming(void)
{
	char	*msg, *p;
	int		size;


	// set calling type
	call_type = CALLEE;

	// if exists display name in INVITE, get name and display to LCD
	getdspname();

	// check timer option.
	//  remote is Supported timer ?
	if((p = stristr(rx_buf, "Supported:")) != NULL && stristr(p, "timer") != NULL){
		timer_opt = 1;
		//  exists Session-Expires and refresher parameter ?
		if((p = stristr(rx_buf, "Session-Expires:")) != NULL && stristr(p, "refresher=uas") != NULL){
			update_sender = 1;
			// when remote unsupport UPDATE, can't use timer option.
			//  IPphone can't use reINVITE refresh message, it's UPDATE only.
			if((p = stristr(rx_buf, "Allow:")) == NULL || stristr(p, "UPDATE") == NULL){
				timer_opt = 0;
				update_sender = 0;
			}
		}else{
			update_sender = 0;
		}
	}else{
		timer_opt = 0;
		// don't support timer
		update_sender = 0;	// not send UPDATE
	}


	// get remote rtp IP/port@
	anlys_sdp();

	// send 100 Trying
	msg = make_100_msg();
	SipMsgSend(msg);

	delay_ms(200);

	// make  180 Ringing( don't use 100rel ) message
	msg = make_180_msg();
	// saved 180 sip msg for 200OK(INVITE)/BYE/UPDATE/487Terminate
	memory_copy((uchar *)saved_msg, (uchar *)msg, MSG_BUF_LEN);
	// send 180
	SipMsgSend(msg);

	// ring bell on
	ring_bell(ON);
	icon_ring_bell(ON);

	for(;;){

		// doing ring bell
		do_ring_bell();

		// check off Hook
		//   send 200, state change to CONNECT
		if(getkey() == KEY_OFF_HOOK){

			// stop bell sound
			ring_bell(OFF);
			icon_ring_bell(OFF);
			icon_telephone(ON);

			// make 200OK message
			memory_copy((uchar *)rx_buf, (uchar *)saved_msg, MSG_BUF_LEN);
			msg = make_200OK_msg(timer_opt, update_sender);

			// 200OK send and wait for ACK(with timer)
			if(send_message_with_retry(msg) == -1){
				tel_state = STATE_DISCONNECT_OFF_HOOK;
			}else{
				tel_state = STATE_CONNECT;
			}
			return;
		}

		// check received CANCEL
		if((size = RcvSipMsg(rx_buf, RX_BUF_LEN)) > 0){
			rx_buf[size] = 0;
			if(strncmp(rx_buf, "CANCEL ", 7) == 0){
				// send 200 ok for CANCEL
				msg = make_200_msg();
				SipMsgSend(msg);

				// INVITE terminate
				memory_copy((uchar *)rx_buf, (uchar *)saved_msg, MSG_BUF_LEN);
				msg = make_487_msg();
				SipMsgSend(msg);

				// bell off
				ring_bell(OFF);
				icon_ring_bell(OFF);

				// change status
				tel_state = STATE_DISCONNECT_ON_HOOK;
				return;

			}else if(strncmp(rx_buf, "INVITE ", 7) == 0){
				// send 486 Busy Here
				msg = make_486_msg();
				SipMsgSend(msg);

			}else{
				// ignore other REQUEST and RESPONSE
				;
			}
		}

	} //for(;;)

}

/*-------------------------------------------------------------------------
	get display name from From Header, and display to LCD
-------------------------------------------------------------------------*/

void	getdspname(void)
{
	char	*np;
	int		len;

	if((np = stristr(rx_buf, "From:")) == NULL) return;
	np += 5;

	lcd_cmdwrite(LCD_SET_LINE2);
	len = 0;
	// skip space
	while(*np == ' ') np++;

	// skip wquot
	if(*np == '"') np++;

	// display to LCD
	while(*np != '<' && *np != '"' && *np != ' '){
		lcd_datawrite(*np++);
		len++;
	}
	if(len == 0) return;	// has no display_name

	// display tail space
	len = LCD_DSP_LENGTH -len;
	while(len > 0){
		lcd_datawrite(' ');
		len--;
	}

}


/*=========================================================================
	state CONNECT
=========================================================================*/

void	st_connect(void)
{
	char	*msg;
	int		size, tid, c;


	// this side is UPDATE sender.
	if(update_sender){
		tid = timer_entry(get_session_time() /2 *10, NULL);
		//tid = timer_entry(SESSION_TIME /2 *10 , NULL);
	}else{
		tid = -1;
	}

	// start voice
	voice(ON);

	for(;;){

		// REGISTER send timing?
		if(reg_tmr != -1 && timer_check(reg_tmr) == 0){
			SndRegister();
			timer_update(reg_tmr, get_expires_time()/2 * 10); // sec -> 0.1sec
			//timer_update(reg_tmr, EXPIRES_TIME /2 * 10); // sec -> 0.1sec
		}

		// UPDATE message send timing?
		if(tid != -1 && timer_Up(tid)){
			SndUpdate();
			timer_update(tid, get_session_time() /2 *10);
			//timer_update(tid, SESSION_TIME /2 *10);
		}

		// doing voice
		do_voice();

		// check key press
		c = getkey();
		if(c == KEY_ON_HOOK){
			SndBye();
			tel_state = STATE_DISCONNECT_ON_HOOK;
			break; // exit for loop
		}else if((c >= '0' && c <= '9') || c == '*' || c == '#' ){
			dtmf_tone(c);
		}

		// doing voice
		do_voice();

		// check received SIP message
		if((size = RcvSipMsg(rx_buf, RX_BUF_LEN)) <= 0) continue;
		rx_buf[size] = 0;

		// received BYE
		if(strncmp(rx_buf, "BYE ", 4) == 0){
			msg = make_200_msg();		
			SipMsgSend(msg);
			tel_state = STATE_DISCONNECT_OFF_HOOK;
			break; // exit for loop

		// received UPDATE
		}else if(strncmp(rx_buf, "UPDATE ", 7) == 0){
			msg = make_200_msg();		
			SipMsgSend(msg);

		// received reINVITE/refer INVITE
		}else if(strncmp(rx_buf, "INVITE ", 7) == 0){
			// get remote rtp IP/port@
			anlys_sdp();
			// send 200 ok
			msg = make_200OK_msg(timer_opt, update_sender);
			SipMsgSend(msg);

		// ignore other request/response 
		}else{
			;
		}
	}

	voice(OFF);
	timer_delete(tid);

}


/*-------------------------------------------------------------------------
	SndBye()
-------------------------------------------------------------------------*/

int		SndBye(void)
{
	char	branch[BRANCH_SIZE], *msg;
	int		cmdseq, size;

	// make message from 180ringing(at Callee) or 200OK(at Caller)
	memory_copy((uchar *)rx_buf, (uchar *)saved_msg, MSG_BUF_LEN);
	make_branch(branch);
	cmdseq = make_seq();
	msg = make_BYE_msg(tel, branch, cmdseq, call_type);

	size = send_message_with_retry(msg);
	if(size == -1) return -1;	// error!! or time out!!
	rx_buf[size] = 0;

	// received 200OK!!
	if(check_200_status(rx_buf, "BYE")) return 0;
	return -1;	// received other message.
}


/*-------------------------------------------------------------------------
	SndUpdate()
-------------------------------------------------------------------------*/

int		SndUpdate(void)
{
	char	branch[BRANCH_SIZE], *msg;
	int		cmdseq, size;

	// make message from 180ringing(at Callee) or 200OK(at Caller)
	memory_copy((uchar *)rx_buf, (uchar *)saved_msg, MSG_BUF_LEN);
	make_branch(branch);
	cmdseq = make_seq();
	msg = make_UPDATE_msg(tel, branch, cmdseq, call_type);

	size = send_message_with_retry(msg);
	if(size == -1) return -1;	// error!! or time out!!
	rx_buf[size] = 0;

	// received 200OK!!
	if(check_200_status(rx_buf, "UPDATE")) return 0;
	return -1;	// received other message.
}


/*-----------------------------------------------------------------------------
	check 200OK status
------------------------------------------------------------------------------*/

int		check_200_status(char *buf, char *method)
{
	char *p, *p1;

	if(strncmp(buf,"SIP/2.0 200", 11) != 0) return 0;
	p = stristr(buf, "CSeq:"); if(p == NULL) return 0;
	if((p1 = stristr(p, method)) == NULL) return 0;
	if(p1 > strstr(p, "\r\n")) return 0;
	return 1;
}


/*=========================================================================
	sending REGISTER
=========================================================================*/

void	send_REGISTER(void)
{
	int		c;

	lcd_cmdwrite(LCD_SET_LINE2);
	lcd_strwrite("send REGISTER ? ");
	lcd_cmdwrite(LCD_SET_LINE2);

	while((c = getkey()) == -1) delay_10us(10);
	if(c == KEY_ENTER){
		lcd_strwrite("sending ...     ");
		lcd_cmdwrite(LCD_SET_LINE2);
		if(SndRegister() == -1){
			lcd_strwrite("Failed REGIST   ");
		}else{
			lcd_strwrite("Success REGIST  ");
		}
		if(reg_tmr == -1){
			reg_tmr = timer_entry(get_expires_time()/2 *10, NULL);
			//reg_tmr = timer_entry(EXPIRES_TIME /2 *10, NULL);
		}else{
			timer_update(reg_tmr, get_expires_time()/2 * 10); 
			//timer_update(reg_tmr, EXPIRES_TIME /2 *10); 
		}
	}else{
		lcd_cmdwrite(cmd_CLEAR);
	}
}


/*------------------------------------------------------------------------------
	SndRegister()
	-1 timeout/fail, 0 success
------------------------------------------------------------------------------*/

int		SndRegister(void)
{
	char	*msg, *expires;
	char	branch[BRANCH_SIZE];
	char	from_tag[TAG_SIZE];
	char	*to_tag;
	char	call_id[CALL_ID_SIZE];
	int		cmdseq;
	int		retry_401;
	int		size;

	// set parameter
	make_branch(branch);
	make_tag(from_tag);
	make_call_id(call_id);
	cmdseq = make_seq();
	to_tag = NULL;
	retry_401 = 0;


	// make first register message
	msg = make_REGISTER_msg(branch, from_tag, to_tag, call_id, cmdseq, NULL);

	// send REGISTER message
msg_send:
	size = send_message_with_retry(msg);
	if(size == -1) return -1;	// error!! or time out!!

	// set buffer end@mark
	rx_buf[size] = 0;

	// find 200OK, 
	if(strncmp(rx_buf, "SIP/2.0 200", 11) == 0 && stristr(rx_buf, "REGISTER") != NULL ){
		expires = strstr(rx_buf, ";expires=");
		if(expires){
			set_expires_time(atoi(expires + strlen(";expires=")));
		}
		return 0;	// REGIST Success !!
	}

	// find 401
	if(strncmp(rx_buf, "SIP/2.0 401", 11) == 0){
		if(retry_401) return -1;	// repeat received 401 error!!
		retry_401++;
 
		make_branch(branch);	// update transaction
		cmdseq = make_seq();	// update sequence number
		make_auth(auth, "REGISTER");
		//to_tag = get_to_tag();

		msg = make_REGISTER_msg(branch, from_tag, to_tag, call_id, cmdseq, auth);
		goto msg_send;
	}

	// received other error response!!
	return -1;

}


/*** end of ua.c **************************************************************/
